/*****************************************************************

  Generic OpenGL routines

  Copyright 1998 by Mike Oliphant - oliphant@ling.ed.ac.uk

    http://www.ling.ed.ac.uk/~oliphant/glmame

  This code may be used and distributed under the terms of the
  Mame license

  Patched for Win32 OpenGL Version
  10/Mai/1999 Sven Goethel, http://www.jausoft.com

*****************************************************************/

#ifdef xgl

#ifdef WIN32
	#define WIN32_LEAN_AND_MEAN
	#include <windows.h>
	#include <windowsx.h>
	#include <assert.h>
	#include <math.h>
	#include "MAME32.h"
	#include "wgl_tool.h"
	#include "wgl_GDIDisplay.h"
#else 
	#include <math.h>
	#include <GL/gl.h>
	#include <GL/glu.h>
	#include "xmame.h"
#endif

#include "driver.h"
#include "glmame.h"
#include "mame.h"

#include <stdio.h>
#include "jpeglib.h"
#include <setjmp.h>

#include "png.h"

#define MAKECOL(r, g, b) (((r) & 0x001F) << 10 | ((g) & 0x001F) << 5 | ((b) & 0x001F))

int LoadCabinet(const char *fname);

#ifndef WIN32
	void SwapBuffers(void);
#endif

void vector_vh_update(struct osd_bitmap *bitmap,int full_refresh);
void vector_clear_list(void);

static void gl_calibrate_palette();

void DeltaVec(GLfloat x1,GLfloat y1,GLfloat z1,
			  GLfloat x2,GLfloat y2,GLfloat z2,
			  GLfloat *dx,GLfloat *dy,GLfloat *dz);
void CalcFlatPoint(int x,int y,GLfloat *px,GLfloat *py);
void CalcCabPoint(int x,int y,GLfloat *px,GLfloat *py,GLfloat *pz);
void SetupFrustum(void);
void SetupOrtho(void);
void InitTextures();
void InitVScreen(void);
void CloseVScreen(void);
void WAvg(GLfloat perc,GLfloat x1,GLfloat y1,GLfloat z1,
		  GLfloat x2,GLfloat y2,GLfloat z2,
		  GLfloat *ax,GLfloat *ay,GLfloat *az);

void UpdateCabDisplay(void);
void UpdateFlatDisplay(void);
void UpdateGLDisplay(void);

void InitCabGlobals();

static void do_save_snapshot (void);
static void	filter_rgb_to_bgr(unsigned char *pixels, int width, int height);

#ifdef WIN32
	PFNGLCOLORTABLEEXTPROC glColorTableEXT = 0;
#endif

static int useGlEXT = 
	#ifdef GL_EXT_paletted_texture
		1
	#else
		0
	#endif
	;

char cabname[512];
int cabspecified=0;
GLint bitmap_lod=0; /* level of display-detail of the bitmap-texture */

extern GLuint cablist;

#ifndef WIN32
	extern int winwidth;
	extern int winheight;
#endif

static double scrnaspect,vscrnaspect;
	
#ifdef GL_EXT_paletted_texture
	GLubyte *ctable=0;			// for OpenGL
#endif
GLushort *rcolmap=0, *gcolmap=0, *bcolmap=0, *acolmap=0;
unsigned char *ctable_orig=0;	// for original return 2 game values
int	       ctable_size;	        /* the true color table size */
GLenum	   gl_bitmap_format;        /* only GL_UNSIGNED_BYTE and 
				                       GL_UNSIGNED_SHORT is supported */
GLint      gl_internal_format;
GLsizei text_width;
GLsizei text_height;

int force_text_width_height = 0;

int screendirty=1;  /* Has the bitmap been modified since the last frame? */

#ifdef WIN32
float gamma_correction = 1.0;
#endif

static int palette_dirty = 0;
static int totalcolors;
unsigned char gl_alpha_value = 255;
static int tx_palette_change=0;
static int frame=0;

extern int fx;
int cabview=0;  /* Do we want a cabinet view or not? */
int cabload_err=0;
int bilinear=1; /* Do binlinear filtering? */
static int drawbitmap=1;
static int dopersist=0;

#ifndef WIN32
	int doublebuffer=1;
	int dodepth=1;
#endif

/* The squares that are tiled to make up the game screen polygon */

struct TexSquare
{
  GLubyte *texture;
  GLuint texobj;
  GLfloat x1,y1,z1,x2,y2,z2,x3,y3,z3,x4,y4,z4;
  GLfloat fx1,fy1,fx2,fy2,fx3,fy3,fx4,fy4;
  GLfloat xcov,ycov;
};

static struct TexSquare *texgrid=0;
static int texnumx;
static int texnumy;
static GLfloat texpercx;
static GLfloat texpercy;
static GLfloat polysize=10.0;
static GLfloat vscrntlx;
static GLfloat vscrntly;
static GLfloat vscrnwidth;
static GLfloat vscrnheight;
static GLfloat xinc,yinc;
static GLint memory_x_len;
static GLint memory_x_start_offset;
static int bytes_per_pixel;


GLfloat cscrx1,cscry1,cscrz1,cscrx2,cscry2,cscrz2,
  cscrx3,cscry3,cscrz3,cscrx4,cscry4,cscrz4;
GLfloat cscrwdx,cscrwdy,cscrwdz;
GLfloat cscrhdx,cscrhdy,cscrhdz;

extern GLubyte *cabimg[5];
extern GLuint cabtex[5];

extern struct CameraPan *cpan;
extern int numpans;
int currentpan=0;
int lastpan=0;
int panframe=0;

/* Vector variables */

extern GLuint veclist;
extern int inlist;
int vecgame=0;

int antialias=0;
int alphablending=0; 

int gl_is_initialized=0;

#ifdef WIN32
int useBMPSnap = 1;
#endif
int usePPMSnap = 
	#ifdef WIN32
		0;
	#else
		1;
	#endif
int useJPGSnap = 0;
int usePNGSnap = 0;
int jpgQuality = 100;

static int do_snapshot = 0;

#ifdef WIN32
	#ifndef NDEBUG
		#define CHECK_WGL_ERROR(a,b,c) check_wgl_error(a,b,c)
		#define CHECK_GL_ERROR(a,b)  check_gl_error(a,b)
	#else
		#define CHECK_WGL_ERROR(a,b,c) 
		#define CHECK_GL_ERROR(a,b)
	#endif
#else
	#ifndef NDEBUG
		#define CHECK_WGL_ERROR(a,b,c)
		#define CHECK_GL_ERROR(a,b)  check_gl_error(a,b)
	#else
		#define CHECK_WGL_ERROR(a,b,c) 
		#define CHECK_GL_ERROR(a,b)
	#endif
#endif

#ifdef WIN32
static void check_wgl_error(HWND wnd, const char * file, int line)
{
	LPVOID lpMsgBuf;
	
	FormatMessage( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER | \
		FORMAT_MESSAGE_FROM_SYSTEM,    0,
		GetLastError(),
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMsgBuf,    0,    0 );// Display the string.

	fprintf(stderr, "%s:%d>%s", file, line, lpMsgBuf);

	// Free the buffer.
	LocalFree( lpMsgBuf );
}
#endif

static void check_gl_error(const char * file, int line)
{
  int ec = glGetError();

  if(ec!=GL_NO_ERROR)
  {
	const char * errstr = gluErrorString(ec);
	if(errstr!=0)
		fprintf(stderr, "%s:%d>0x%X %s", file, line, ec, errstr);
	else
		fprintf(stderr, "%s:%d>0x%X <unknown>", file, line, ec);
  }		   
}


static void fetchGL_EXT_FUNCS()
{
  if(glColorTableEXT) return;

#ifdef WIN32
#ifdef GL_EXT_paletted_texture
  glColorTableEXT = (PFNGLCOLORTABLEEXTPROC) 
	  wglGetProcAddress("glColorTableEXT");
  if(glColorTableEXT == 0 )
  {
	  CHECK_WGL_ERROR(glWnd, __FILE__, __LINE__);
	  useGlEXT = 0;
  } 
#endif
#endif

}

int glHasEXT(void)
{
	#ifdef GL_EXT_paletted_texture
		return 1;
	#else
		return 0;
	#endif
}

void glSetUseEXT(int val)
{
	#ifdef GL_EXT_paletted_texture
		if(gl_is_initialized==0)
			useGlEXT=val;
	#endif
}

int glGetUseEXT(void)
{
	return useGlEXT;
}

/* Set up the virtual screen */

void InitVScreen(void)
{
  /* clear the buffer */

  if(glContext==0) return;

  fetchGL_EXT_FUNCS();

  InitCabGlobals();

  glClearColor(0,0,0,1);
  glClear(GL_COLOR_BUFFER_BIT);
  glFlush();

  if(!dodepth) cabview=0;

  currentpan=0;
  lastpan=0;
  panframe=0;

  scrnaspect=(double)visual_width/(double)visual_height;
  vscrnaspect=(double)winwidth/(double)winheight;

  if(scrnaspect<vscrnaspect) {
	vscrnheight=(GLfloat)winheight;
	vscrnwidth=vscrnheight*scrnaspect;
	vscrntlx=((GLfloat)winwidth-vscrnwidth)/2.0;
	vscrntly=0.0;
  }
  else {
	vscrnwidth=(GLfloat)winwidth;
	vscrnheight=vscrnwidth/scrnaspect;
	vscrntlx=0.0;
	vscrntly=((GLfloat)winheight-vscrnheight)/2.0;
  }

  glViewport(0,0,winwidth,winheight);

  if(dodepth) glDepthFunc(GL_LEQUAL);

#ifdef WIN32
  __try 
  { 
#endif
		if(!cabspecified || !LoadCabinet(cabname) ) 
		{
			if(cabspecified)
				printf("Unable to load cabinet %s\n",cabname);
			strcpy(cabname,"glmame");
			cabspecified=1;
			LoadCabinet(cabname);
			cabload_err=0;
		}

#ifdef WIN32
  } __except( GetExceptionCode() ) 
  {
        fprintf(stderr, "\nCabinet loading is still buggy - sorry !\n");    
		cabload_err=1;
  }
#endif

  /* Calulate delta vectors for screen height and width */

  DeltaVec(cscrx1,cscry1,cscrz1,cscrx2,cscry2,cscrz2,
		   &cscrwdx,&cscrwdy,&cscrwdz);
	
  DeltaVec(cscrx1,cscry1,cscrz1,cscrx4,cscry4,cscrz4,
		   &cscrhdx,&cscrhdy,&cscrhdz);
	

  if(Machine->drv->video_attributes & VIDEO_TYPE_VECTOR)
  {
	vecgame=1;
	alphablending=1;
  }
  else
	vecgame=0;
}

/* original : */
/*
    palette is an array of 'totalcolors' R,G,B triplets. The function returns
    in *pens the pen values corresponding to the requested colors.
    If 'totalcolors' is 32768, 'palette' is ignored and the *pens array is filled
    with pen values corresponding to a 5-5-5 15-bit palette
*/
/* my true color test :-) : 
	
   now - we do use true size of the color table .. JAU 
*/

int
#ifdef WIN32
	gl_allocate_colors
#else
	osd_allocate_colors
#endif
	(unsigned int _totalcolors, const unsigned char *palette, unsigned short *pens, int modifiable)
{
   char buf[1000];
   int col;
   int bestblack;
   int bestwhite;
   int bestblackscore;
   int bestwhitescore;
   int r, g, b, score;

   if(glContext==0) return 1;

   totalcolors = _totalcolors;

   if (totalcolors>256)
   {
      sprintf(buf, "Warning: More than 256 colors (%d) are needed for this emulation,\n%s",
                   totalcolors, "Using the full pallete within OpenGL-test-code ! \n" );
	  fprintf(stderr, buf);
   }

  if(bitmap->depth!=8 && bitmap->depth!=16)
  {
	  int new_deepth=0;
	  if(bitmap->depth<8)
		 new_deepth=8;
	  else 
		 new_deepth=16;

      sprintf(buf, "ERROR: %d color-index bitmap deepth impossible for now, using new deepth: %d\n",
                   bitmap->depth, new_deepth);
	  fprintf(stderr, buf);
	  bitmap->depth=new_deepth;
  }

  /* find a OpenGL compatible colortablesize=2**n */
  ctable_size=1;
  while(ctable_size<totalcolors)
	  ctable_size*=2;

  if(bitmap->depth==8)
  {
	gl_bitmap_format=GL_UNSIGNED_BYTE;
	gl_internal_format=GL_COLOR_INDEX8_EXT;
  }
  else
  {
	gl_bitmap_format=GL_UNSIGNED_SHORT;
	gl_internal_format=GL_COLOR_INDEX16_EXT;
  }
  if(useGlEXT==0)
  {
	  if(alphablending)
	  {
	    gl_internal_format=GL_RGBA;
	  } else {
	    gl_internal_format=GL_RGB;
	  }
  }

#ifdef GL_EXT_paletted_texture
  if(useGlEXT && glColorTableEXT)
  {
	 if(alphablending)
		ctable=(GLubyte *)calloc(ctable_size * 4, 1);
	 else
		ctable=(GLubyte *)calloc(ctable_size * 3, 1);
  }
  else 
  {
#endif
	rcolmap=(GLushort *)calloc(ctable_size*sizeof(GLushort), 1);
	gcolmap=(GLushort *)calloc(ctable_size*sizeof(GLushort), 1);
	bcolmap=(GLushort *)calloc(ctable_size*sizeof(GLushort), 1);
	if(alphablending)
	  acolmap=(GLushort *)calloc(ctable_size*sizeof(GLushort), 1);
	else
	  acolmap=0;
#ifdef GL_EXT_paletted_texture
  }
#endif


  ctable_orig=(unsigned char *)malloc(ctable_size * 3);

  /* init the palette - If 'totalcolors' is NOT 32768 only the pen is */   
  bestblack = bestwhite = 0;
  bestblackscore = 3 * 255 * 255;
  bestwhitescore = 0;

  #ifndef NDEBUG
	  fprintf(stderr, "totalcolors = %d, deepth = %d \n", 
		  totalcolors, bitmap->depth);

	  fprintf(stderr, "pens[0..%d]: \n\t", totalcolors-1);
  #endif

  for (col=0;  col<totalcolors; col++)
  {
	   r=0; g=0; b=0;
	   if(totalcolors<32768)
	   {
			r = palette[col*3+0];
			g = palette[col*3+1];
			b = palette[col*3+2];
			score = r * r + g * g + b * b;

			if (score < bestblackscore)
			{
				bestblack      = col;
				bestblackscore = score;
			}
			if (score > bestwhitescore)
			{
				bestwhite      = col;
				bestwhitescore = score;
			}

			pens[col] = col;
	   } else {
			pens[col] = (totalcolors-1)-col;
		    bestblack=0;
		    bestwhite=1;
	   }
	   #ifdef WIN32
			gl_modify_pen
	   #else
			osd_modify_pen
	   #endif
			 (col, r, g, b);
	   #ifndef NDEBUG
		   fprintf(stderr, "%d (%d/%d/%d), ", pens[col], r,g,b);
		   if(col%8==0)
			fprintf(stderr, "\n\t");
	   #endif
  }

  if(bestblack==bestwhite)
	 bestwhite=1;
   
  /* map black to pen 0, otherwise the screen border will not be black */
  pens[bestblack] = 0;
  pens[0] = bestblack;

  #ifdef WIN32
	gl_modify_pen
  #else
	osd_modify_pen
  #endif
	 (bestblack, 0, 0, 0);
  #ifdef WIN32
	gl_modify_pen
  #else
	osd_modify_pen
  #endif
	 (bestwhite, 0xff, 0xff, 0xff);

  Machine->uifont->colortable[0] = bestblack;
  Machine->uifont->colortable[1] = bestwhite;
  Machine->uifont->colortable[2] = bestwhite;
  Machine->uifont->colortable[3] = bestblack;

  #ifndef NDEBUG
	  fprintf(stderr, "\n");

	  fprintf(stderr, "bestblack = %d, bestwhite = %d \n", 
		bestblack, bestwhite);
  #endif

  palette_dirty = 1;

  InitTextures();

  return 0;
}


void InitTextures()
{
  int e_x, e_y, s, i;
  int x,y, glerrid;
  GLuint *tobj;
  GLint format;
  unsigned char * line_1, *line_2, *mem_start;
  struct TexSquare *tsq;

   if(glContext==0) return;

  /* lets clean up ... hmmm ... */
  glEnd();
  glerrid = glGetError();
  if(0x502!=glerrid)
  {
	  CHECK_GL_ERROR(__FILE__, __LINE__);
  }

  texnumx=1;
  texnumy=1;

  /* achieve the 2**e_x:=text_width, 2**e_y:=text_height */
  e_x=0; s=1;

  while (s<text_width)
  { s*=2; e_x++; }

  e_y=0; s=1;

  while (s<text_height)
  { s*=2; e_y++; }
  
  /* Test the max texture size */
  do {
		glTexImage2D(GL_PROXY_TEXTURE_2D, bitmap_lod, 
			         gl_internal_format,
					 text_width, text_height, 
					 0,
					 GL_COLOR_INDEX, gl_bitmap_format, 0);
		glGetTexLevelParameteriv
					(GL_PROXY_TEXTURE_2D, bitmap_lod,
					 GL_TEXTURE_INTERNAL_FORMAT, &format);

		if(format==gl_internal_format &&
           force_text_width_height>0 &&
		   ( force_text_width_height<text_width ||
		     force_text_width_height<text_height
		   )
		  )
		{
			format=0;
		}

		if(format!=gl_internal_format)
		{
			if(text_width>text_height) 
			{
				e_x--; text_width=1;
				for(i=e_x; i>0; i--)
					text_width*=2;
			} else {
				e_y--; text_height=1;
				for(i=e_y; i>0; i--)
					text_height*=2;
			}
		}
  } while (format!=gl_internal_format && 
		   text_width>1 && text_height>1);

  texnumx = visual_width / text_width;
  if( (visual_width % text_width)>0 ) texnumx++;

  texnumy = visual_height / text_height;
  if( (visual_height % text_height)>0 ) texnumy++;

  /* Allocate the texture memory */
  
  xinc=vscrnwidth*((double)text_width/(double)visual_width);
  yinc=vscrnheight*((double)text_height/(double)visual_height);
  
  texpercx=(GLfloat)text_width/(GLfloat)visual_width;
  if(texpercx>1.0) texpercx=1.0;
  
  texpercy=(GLfloat)text_height/(GLfloat)visual_height;
  if(texpercy>1.0) texpercy=1.0;
  
  texgrid=(struct TexSquare *)
	malloc(texnumx*texnumy*sizeof(struct TexSquare));
   
  bytes_per_pixel = bitmap->depth/8;

  for(y=0;y<texnumy;y++) {
	for(x=0;x<texnumx;x++) {
	  tsq=texgrid+y*texnumx+x;
	  
	  if(x==texnumx-1 && visual_width%text_width)
		tsq->xcov=(GLfloat)(visual_width%text_width)/(GLfloat)text_width;
	  else tsq->xcov=1.0;
	  
	  if(y==texnumy-1 && visual_height%text_height)
		tsq->ycov=(GLfloat)(visual_height%text_height)/(GLfloat)text_height;
	  else tsq->ycov=1.0;
	  
	  CalcCabPoint(x,y,&(tsq->x1),&(tsq->y1),&(tsq->z1));
	  CalcCabPoint(x+1,y,&(tsq->x2),&(tsq->y2),&(tsq->z2));
	  CalcCabPoint(x+1,y+1,&(tsq->x3),&(tsq->y3),&(tsq->z3));
	  CalcCabPoint(x,y+1,&(tsq->x4),&(tsq->y4),&(tsq->z4));

	  CalcFlatPoint(x,y,&(tsq->fx1),&(tsq->fy1));
	  CalcFlatPoint(x+1,y,&(tsq->fx2),&(tsq->fy2));
	  CalcFlatPoint(x+1,y+1,&(tsq->fx3),&(tsq->fy3));
	  CalcFlatPoint(x,y+1,&(tsq->fx4),&(tsq->fy4));

	  /* calculate the pixel store data,
         to use the machine-bitmap for our texture 
      */
	  line_1	= (unsigned char *) 
			  bitmap->line[visual.min_y+y*text_height];

	  line_2	= (unsigned char *) 
			  bitmap->line[visual.min_y+y*text_height+1];

	  mem_start = (unsigned char *) bitmap->_private;

	  memory_x_len			=	(line_2 - line_1) /
								bytes_per_pixel;

	  memory_x_start_offset	=   visual.min_x*bytes_per_pixel +
					            x * text_width;

	  tsq->texture = line_1 +
				    memory_x_start_offset;

	  if(options.errorlog!=0 && x==0 && y==0)
	  {
		fprintf(stderr, "bitmap (w=%d / h=%d / d=%d)\n", 
			bitmap->width, bitmap->height, bitmap->depth);

		fprintf(stderr, "visual (w=%d / h=%d)\n", 
			visual_width, visual_height);

		fprintf(stderr, "win (w=%d / h=%d)\n", 
			winwidth, winheight);

		fprintf(stderr, "game (w=%d / h=%d)\n", 
			Machine->drv->visible_area.max_x-Machine->drv->visible_area.min_x+1,
			Machine->drv->visible_area.max_y-Machine->drv->visible_area.min_y+1);

		fprintf(stderr, "texture-usage %d*width=%d, %d*height=%d\n",
		    (int)texnumx, (int)text_width, (int)texnumy, (int)text_height);

		fprintf(stderr, "texture-usage colors=(%d/%d)\n", 
		    (int)totalcolors, (int)ctable_size);

		fprintf(stderr, "row_len=%d, x_ofs=%d, bytes_per_pixel=%d, 1st pix. ofs=%d\n", 
			(int)memory_x_len, (int)memory_x_start_offset, 
			(int)bytes_per_pixel,
			(int) 
			(
				( mem_start - (line_1 + memory_x_start_offset) ) * 
				bytes_per_pixel 
			)
		);
	  }

      tobj=&(tsq->texobj);
	  
	  glGenTextures(1,tobj);

	  glBindTexture(GL_TEXTURE_2D,*tobj);
	  	  
	  #ifdef GL_EXT_paletted_texture
		if(useGlEXT && glColorTableEXT) 
		{
			if(alphablending)
			{
				glColorTableEXT(GL_TEXTURE_2D,
					GL_RGBA,
					ctable_size,
					GL_RGBA,
					GL_UNSIGNED_BYTE,
					ctable);
			} else {
				glColorTableEXT(GL_TEXTURE_2D,
					GL_RGB,
					ctable_size,
					GL_RGB,
					GL_UNSIGNED_BYTE,
					ctable);
			}
	  
			CHECK_GL_ERROR( __FILE__, __LINE__);
		} else {
	  #endif
			glPixelMapusv(GL_PIXEL_MAP_I_TO_R,ctable_size,rcolmap);
			glPixelMapusv(GL_PIXEL_MAP_I_TO_G,ctable_size,gcolmap);
			glPixelMapusv(GL_PIXEL_MAP_I_TO_B,ctable_size,bcolmap);
			if(alphablending)
				glPixelMapusv(GL_PIXEL_MAP_I_TO_A,ctable_size,acolmap);
	  #ifdef GL_EXT_paletted_texture
		}
	  #endif

      glTexImage2D(GL_TEXTURE_2D,bitmap_lod,
		           gl_internal_format,
				   text_width,text_height,
		           0,
	       	       GL_COLOR_INDEX,gl_bitmap_format, 0);

	  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	  
	  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
	  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
  
	  glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);

	} /* for all texnumx*/
  } /* for all texnumy*/


  gl_is_initialized=1;

  /* lets init the rest of the custumizings ... */
  gl_set_bilinear(bilinear);
  gl_set_cabview(cabview);
  gl_set_antialias(antialias);
  gl_set_alphablending(alphablending);

  CHECK_GL_ERROR( __FILE__, __LINE__);

}

int gl_GetBlackPen(void)
{
    if (bitmap->depth == 8)
    {
        int i, cofs_orig;

        for (i = 0; i < ctable_size; i++)
        {
			cofs_orig=i*3;
			if( ctable_orig[cofs_orig]	== 0 &&
				ctable_orig[cofs_orig+1]== 0 &&
				ctable_orig[cofs_orig+2]== 0 
			  )
            {
                return i;
            }
        }
    }

    return 0;
}

static void gl_calibrate_palette()
{
  int col;

  if(gl_is_initialized==0) return;

  #ifndef NDEBUG
	   fprintf(stderr, "gl-palette calibrate [0..%d]: \n\t", totalcolors-1);
  #endif
  for (col=0; col<totalcolors; col++)
  {
		int cofs;
		int cofs_orig=col*3;

		unsigned char red=ctable_orig[cofs_orig], 
			      green=ctable_orig[cofs_orig+1], 
			      blue=ctable_orig[cofs_orig+2];

#ifdef GL_EXT_paletted_texture
	if(useGlEXT && glColorTableEXT)
	{
		double dAlpha = (double) gl_alpha_value / 255.0 ;

	    if(alphablending)
		{
			cofs=col*4;
			ctable[cofs]= 255  * pow(red   / 255.0, 1 / gamma_correction);

			ctable[cofs+1]=255 * pow(green / 255.0, 1 / gamma_correction);

			ctable[cofs+2]=255 * pow(blue  / 255.0, 1 / gamma_correction);

			ctable[cofs+3]=gl_alpha_value;
			#ifndef NDEBUG
				   fprintf(stderr, "%d (%d/%d/%d/%d), ", 
					   col, ctable[cofs], ctable[cofs+1], ctable[cofs+2], ctable[cofs+3]);
				   if(col%8==0)
					fprintf(stderr, "\n\t");
			#endif
		} else {
			cofs=col*3;
			ctable[cofs]=  255 * (dAlpha * pow(red   / 255.0, 1 / gamma_correction) );

			ctable[cofs+1]=255 * (dAlpha * pow(green / 255.0, 1 / gamma_correction) );

			ctable[cofs+2]=255 * (dAlpha * pow(blue  / 255.0, 1 / gamma_correction) );
			#ifndef NDEBUG
				   fprintf(stderr, "%d (%d/%d/%d), ", 
					   col, ctable[cofs], ctable[cofs+1], ctable[cofs+2]);
				   if(col%8==0)
					fprintf(stderr, "\n\t");
			#endif
		}
	} else {
#endif
	    if(alphablending)
		{
		  cofs=col;
		  rcolmap[cofs]=65535 * pow(red   / 255.0, 1 / gamma_correction);

		  gcolmap[cofs]=65535 * pow(green / 255.0, 1 / gamma_correction);

		  bcolmap[cofs]=65535 * pow(blue  / 255.0, 1 / gamma_correction);

		  acolmap[cofs]=65535 * (gl_alpha_value / 255.0);
		} else {
		  double dAlpha = (double) gl_alpha_value / 255.0 ;

		  cofs=col;
		  rcolmap[cofs]=65535 * (dAlpha * pow(red   / 255.0, 1 / gamma_correction));

		  gcolmap[cofs]=65535 * (dAlpha * pow(green / 255.0, 1 / gamma_correction));

		  bcolmap[cofs]=65535 * (dAlpha * pow(blue  / 255.0, 1 / gamma_correction));
		}
#ifdef GL_EXT_paletted_texture
	}
#endif
  }
}

/* Change the color of a pen */
void
#ifdef WIN32
	gl_modify_pen
#else
	osd_modify_pen
#endif
	(int pen, unsigned char red,unsigned char green, unsigned char blue) 
{
  int cofs_orig;

  if(glContext==0) return;

  /* now we check the true values ;-) JAU */
  if(pen>=ctable_size) 
  {
    char buf[1000];
	sprintf(buf, "gl_modify_pen: pen (%d) is greater than colors in colortable(%d)", 
		pen, ctable_size);
	fprintf(stderr,buf);
  }

  cofs_orig=pen*3;

  ctable_orig[cofs_orig]= red;
  ctable_orig[cofs_orig+1]=green;
  ctable_orig[cofs_orig+2]=blue;

  palette_dirty = 1;
}

void
#ifdef WIN32
	gl_get_pen
#else
	osd_get_pen
#endif
	(int pen, unsigned char* pRed, unsigned char* pGreen, unsigned char* pBlue)
{
  int cofs_orig;

  if(glContext==0) return;

  /* now we check the true values ;-) JAU */
  if(pen>=ctable_size) 
  {
    char buf[1000];
	sprintf(buf, "gl_get_pen: pen (%d) is greater than colors in colortable(%d)", 
		pen, ctable_size);
	fprintf(stderr,buf);
  }

  cofs_orig=pen*3;

  *pRed   = ctable_orig[cofs_orig];
  *pGreen = ctable_orig[cofs_orig+1];
  *pBlue  = ctable_orig[cofs_orig+2];
}

void 
#ifdef WIN32
	gl_set_gamma
#else
	osd_set_gamma
#endif
	(float _gamma)
{
	if(_gamma==0.0) _gamma=0.1;
	gamma_correction = _gamma;

	palette_dirty = 1;
}

float
#ifdef WIN32
	gl_get_gamma
#else
	osd_get_gamma
#endif
	(void)
{
	return gamma_correction;
}

void gl_set_alpha (unsigned char _alpha)
{
	gl_alpha_value=_alpha;

  palette_dirty = 1;
}


unsigned char gl_get_alpha(void)
{
	return gl_alpha_value;
}

/* for compatibility issues ... uses alpha */
void 
#ifdef WIN32
	gl_set_brightness
#else
	osd_set_brightness
#endif
	(int _brightness)
{
	gl_set_alpha((unsigned char)((_brightness/100.0)*255.0));
}

int
#ifdef WIN32
	gl_get_brightness
#else
	osd_get_brightness
#endif
	(void)
{
	return (int) ((gl_get_alpha()/255.0)*100.0);
}



void 
#ifdef WIN32
	gl_mark_dirty
#else
	osd_mark_dirty
#endif
    (int x1, int y1, int x2, int y2, int ui)
{
	screendirty=1;
}

/* Compute a delta vector between two points */

void DeltaVec(GLfloat x1,GLfloat y1,GLfloat z1,
			  GLfloat x2,GLfloat y2,GLfloat z2,
			  GLfloat *dx,GLfloat *dy,GLfloat *dz)
{
  *dx=x2-x1;
  *dy=y2-y1;
  *dz=z2-z1;
}

/* Calculate points for flat screen */

void CalcFlatPoint(int x,int y,GLfloat *px,GLfloat *py)
{
  *px=(float)x*texpercx;

  *py=(float)y*texpercy;
}

/* Calculate points for cabinet screen */

void CalcCabPoint(int x,int y,GLfloat *px,GLfloat *py,GLfloat *pz)
{
  GLfloat xperc,yperc;

  xperc=x*texpercx;
  if(xperc>1.0) xperc=1.0;

  yperc=y*texpercy;
  if(yperc>1.0) yperc=1.0;

  *px=cscrx1+xperc*cscrwdx+yperc*cscrhdx;
  *py=cscry1+xperc*cscrwdy+yperc*cscrhdy;
  *pz=cscrz1+xperc*cscrwdz+yperc*cscrhdz;
}


/* Set up a frustum projection */

void SetupFrustum(void)
{
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glFrustum(-vscrnaspect,vscrnaspect,-1.0,1.0,5.0,100.0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glTranslatef(0.0,0.0,-20.0);
}


/* Set up an orthographic projection */

void SetupOrtho(void)
{
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  glOrtho(-vscrntlx/vscrnwidth,                (vscrnwidth+vscrntlx)/vscrnwidth,
          (vscrnheight-vscrntly)/vscrnheight,  vscrntly/vscrnheight,
		  -1.0, 1.0 );

  #ifndef NDEBUG
	  fprintf(stderr, "view  [p0,p1]: %d/%d, %d/%d\n", (int)vscrntlx, (int)vscrntly, (int)vscrnwidth, (int)vscrnheight);
	  fprintf(stderr, "ortho [lrbt] : %f/%f, %f/%f\n", 
				-vscrntlx/vscrnwidth,                (vscrnwidth+vscrntlx)/vscrnwidth,
				(vscrnheight-vscrntly)/vscrnheight, vscrntly/vscrnheight);
  #endif


  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}

/* Close down the virtual screen */

void CloseVScreen(void)
{
  if(gl_is_initialized==0) return;

#ifdef GL_EXT_paletted_texture
  if(ctable) free(ctable); ctable=0;
#endif
  if(rcolmap) free(rcolmap); rcolmap=0;
  if(bcolmap) free(bcolmap); bcolmap=0;
  if(gcolmap) free(gcolmap); gcolmap=0;
  if(acolmap) free(acolmap); acolmap=0;

  if(cpan) { free(cpan); cpan=0; }

  /* Free Texture stuff */

  if(texgrid) {
	free(texgrid); texgrid=0;
  }

  gl_is_initialized=0;
}


/* Not needed under GL */
#ifndef WIN32
void sysdep_clear_screen(void)
{
}
#endif


/* Compute an average between two sets of 3D coordinates */

void WAvg(GLfloat perc,GLfloat x1,GLfloat y1,GLfloat z1,
		  GLfloat x2,GLfloat y2,GLfloat z2,
		  GLfloat *ax,GLfloat *ay,GLfloat *az)
{
  *ax=(1.0-perc)*x1+perc*x2;
  *ay=(1.0-perc)*y1+perc*y2;
  *az=(1.0-perc)*z1+perc*z2;
}



void drawTextureDisplay(int useCabinet, int updateTexture)
{
    struct TexSquare *square;
	int x,y, wd, ht;
	static const float z_pos = 0.9f;

	if(updateTexture)
	{
		glPixelStorei(GL_UNPACK_ROW_LENGTH , memory_x_len);
		glPixelStorei(GL_UNPACK_ALIGNMENT, 8);
		#ifdef GL_EXT_paletted_texture
			if(!useGlEXT) 
		#endif
				glPixelTransferi(GL_MAP_COLOR, GL_TRUE);
	}
	
	glEnable(GL_TEXTURE_2D);
 
	for(y=0;y<texnumy;y++) {
	  for(x=0;x<texnumx;x++) {
		square=texgrid+y*texnumx+x;

		glBindTexture(GL_TEXTURE_2D,square->texobj);

		if(tx_palette_change) {

			#ifdef GL_EXT_paletted_texture
				if(useGlEXT && glColorTableEXT) 
				{
					if(alphablending)
					{
						glColorTableEXT(GL_TEXTURE_2D,
							GL_RGBA,
							ctable_size,
							GL_RGBA,
							GL_UNSIGNED_BYTE,
							ctable);
					} else {
						glColorTableEXT(GL_TEXTURE_2D,
							GL_RGB,
							ctable_size,
							GL_RGB,
							GL_UNSIGNED_BYTE,
							ctable);
					}
				} else {
			#endif
				glPixelMapusv(GL_PIXEL_MAP_I_TO_R,ctable_size,rcolmap);
				glPixelMapusv(GL_PIXEL_MAP_I_TO_G,ctable_size,gcolmap);
				glPixelMapusv(GL_PIXEL_MAP_I_TO_B,ctable_size,bcolmap);
				if(alphablending)
					glPixelMapusv(GL_PIXEL_MAP_I_TO_A,ctable_size,acolmap);
			#ifdef GL_EXT_paletted_texture
				}
			#endif

		    tx_palette_change=0;

		}

		/* This is the quickest way I know of to update the texture */
		if(updateTexture)
		{
			if(x<texnumx-1)
				wd=text_width;
			else
				wd=visual_width-text_width*x;

			if(y<texnumy-1)
				ht=text_height;
			else
				ht=visual_height-text_height*y;

			glTexSubImage2D(GL_TEXTURE_2D, bitmap_lod,0,0,
							wd, ht,
							GL_COLOR_INDEX, gl_bitmap_format,
							square->texture);
		}

		if(useCabinet)
		{
			glBegin(GL_QUADS);
			glTexCoord2f(0,0); glVertex3f(square->x1,square->y1,square->z1);
			glTexCoord2f(square->xcov,0);
			glVertex3f(square->x2,square->y2,square->z2);
			glTexCoord2f(square->xcov,square->ycov);
			glVertex3f(square->x3,square->y3,square->z3);
			glTexCoord2f(0,square->ycov);
			glVertex3f(square->x4,square->y4,square->z4);
			glEnd();
		} else {
			glBegin(GL_QUADS);
			glTexCoord2f(0,0); glVertex3f(square->fx1,square->fy1, z_pos);
			glTexCoord2f(square->xcov,0);
			glVertex3f(square->fx2,square->fy2, z_pos);
			glTexCoord2f(square->xcov,square->ycov);
			glVertex3f(square->fx3,square->fy3, z_pos);
			glTexCoord2f(0,square->ycov);
			glVertex3f(square->fx4,square->fy4, z_pos);
			glEnd();
		}
	  }
	}

	if(updateTexture)
	{
		glPixelStorei(GL_UNPACK_ROW_LENGTH , 0);
		glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
		#ifdef GL_EXT_paletted_texture
			if(!useGlEXT) 
		#endif
				glPixelTransferi(GL_MAP_COLOR, GL_FALSE);
	}

    glDisable(GL_TEXTURE_2D);
}


/* Draw a frame in Cabinet mode */

void UpdateCabDisplay(void)
{
  int glerrid;
  GLfloat camx,camy,camz;
  GLfloat dirx,diry,dirz;
  GLfloat normx,normy,normz;
  GLfloat perc;
  struct CameraPan *pan,*lpan;

  if(gl_is_initialized==0) return;

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  CHECK_GL_ERROR( __FILE__, __LINE__);

  glPushMatrix();


  /* Do the camera panning */

  if(cpan) {
	pan=cpan+currentpan;

	if(panframe>pan->frames) {
	  lastpan=currentpan;
	  currentpan=(currentpan+1)%numpans;
	  panframe=0;
	}

	switch(pan->type) {
	case pan_goto:
	  camx=pan->lx; camy=pan->ly; camz=pan->lz;
	  dirx=pan->px; diry=pan->px; dirz=pan->pz;
	  normx=pan->nx; normy=pan->ny; normz=pan->nz;
	  break;
	case pan_moveto:
	  lpan=cpan+lastpan;
	  perc=(GLfloat)panframe/(GLfloat)pan->frames;
	  WAvg(perc,lpan->lx,lpan->ly,lpan->lz,
		   pan->lx,pan->ly,pan->lz,&camx,&camy,&camz);
	  WAvg(perc,lpan->px,lpan->py,lpan->pz,
		   pan->px,pan->py,pan->pz,&dirx,&diry,&dirz);
	  WAvg(perc,lpan->nx,lpan->ny,lpan->nz,
		   pan->nx,pan->ny,pan->nz,&normx,&normy,&normz);
	  break;
	default:
	  break;
	}

	gluLookAt(camx,camy,camz,
			  dirx,diry,dirz,
			  normx,normy,normz);
	
	panframe++;
  }
  else gluLookAt(-5.0,0.0,5.0,0.0,0.0,-5.0,0.0,1.0,0.0);

  glEnable(GL_DEPTH_TEST);

  /* Draw the cabinet */
  glCallList(cablist);

  /* YES - lets catch this error ...*/
  glerrid = glGetError();
  if(0x502!=glerrid)
  {
	CHECK_GL_ERROR( __FILE__, __LINE__);
  }

  /* Draw the screen if in vector mode */

  if(vecgame) {
	if(drawbitmap && screendirty ) {
	  // glClear(GL_DEPTH_BUFFER_BIT);

	  // SetupOrtho();
	  
	  glColor4f(1.0,1.0,1.0,0.7);
	  
	  drawTextureDisplay(1 /*cabinet*/, 1);
	  
	  // SetupFrustum();
	} 
	glDisable(GL_TEXTURE_2D);
	glShadeModel(GL_FLAT);

	if(antialias) {
	  glEnable(GL_LINE_SMOOTH);
	  glEnable(GL_POINT_SMOOTH);
	}

	glCallList(veclist);

	if(antialias) {
	  glDisable(GL_LINE_SMOOTH);
	  glDisable(GL_POINT_SMOOTH);
	}

	glShadeModel(GL_SMOOTH);

	/*
	if(drawbitmap && screendirty ) {
	  glClear(GL_DEPTH_BUFFER_BIT);

	  SetupOrtho();
	  
	  glColor4f(1.0,1.0,1.0,0.7);
	  
	  drawTextureDisplay(1, 1);
	  
	  SetupFrustum();
	} */
  }
  else {  /* Draw the screen of a bitmapped game */

	drawTextureDisplay(1 /*cabinet*/, 1);

  }

  glDisable(GL_DEPTH_TEST);

  glPopMatrix();

  if(do_snapshot)
  {
	  do_save_snapshot();
	  do_snapshot=0;
  }

  if(doublebuffer) {
	#ifdef WIN32
	  BOOL ret = SwapBuffers(glHDC);
	  if(ret!=TRUE) {
		CHECK_WGL_ERROR(glWnd, __FILE__, __LINE__);
		doublebuffer=FALSE;
	  }
	#else
	  SwapBuffers();
	#endif
  }
  else glFlush();

  CHECK_GL_ERROR( __FILE__, __LINE__);

}

void UpdateFlatDisplay(void)
{
  int glerrid;

  if(gl_is_initialized==0) return;

  if(!vecgame||!dopersist) glClear(GL_COLOR_BUFFER_BIT);

  glPushMatrix();

  /* lets clean up ... hmmm ... */
  glEnd();
  glerrid = glGetError();
  if(0x502!=glerrid)
  {
	  CHECK_GL_ERROR( __FILE__, __LINE__);
  }
  if(dopersist&&vecgame) {
	glColor4f(0.0,0.0,0.0,0.2);

	glBegin(GL_QUADS);
	glVertex2f(0.0,0.0);
	glVertex2f((GLfloat)winwidth,0.0);
	glVertex2f((GLfloat)winwidth,(GLfloat)winheight);
	glVertex2f(0.0,(GLfloat)winheight);
	glEnd();
  }

  CHECK_GL_ERROR( __FILE__, __LINE__);


  /*if(!vecgame||(drawbitmap && screendirty)) {
	glColor4f(1.0,1.0,1.0,1.0);

	drawTextureDisplay(0);
  }*/

  glColor4f(1.0,1.0,1.0,1.0);

  drawTextureDisplay(0, !vecgame||(drawbitmap && screendirty));

  if(vecgame) {
	glShadeModel(GL_FLAT);

	if(antialias) {
	  glEnable(GL_LINE_SMOOTH);
	  glEnable(GL_POINT_SMOOTH);
	}

	glCallList(veclist);

	if(antialias) {
	  glDisable(GL_LINE_SMOOTH);
	  glDisable(GL_POINT_SMOOTH);
 	}

	glShadeModel(GL_SMOOTH);

    /* YES - lets catch this error ... 
     */
    glerrid = glGetError();
    if(0x502!=glerrid)
	{
	  CHECK_GL_ERROR( __FILE__, __LINE__);
	}
  }

  /*if(!vecgame||(drawbitmap && screendirty)) {
	glColor4f(1.0,1.0,1.0,1.0);

	drawTextureDisplay(0);
  }*/

  glPopMatrix();

  CHECK_GL_ERROR( __FILE__, __LINE__);

  if(do_snapshot)
  {
	  do_save_snapshot();
	  do_snapshot=0;
  }

  if(doublebuffer) {
	#ifdef WIN32
	  BOOL ret = SwapBuffers(glHDC);
	  if(ret!=TRUE) {
		CHECK_WGL_ERROR(glWnd, __FILE__, __LINE__);
		doublebuffer=FALSE;
	  }
	#else
	  SwapBuffers();
	#endif
  }
  else glFlush();

}

void UpdateGLDisplay(void)
{
  int glerrid;

  if(gl_is_initialized==0) return;

  if(vecgame&&inlist) 
  {
	vector_vh_update(0,0);

    /* YES - lets catch this error ... 
	   the upper function calls glEnd at the end, tsts */
    glerrid = glGetError();
    if(0x502!=glerrid)
	{
	  CHECK_GL_ERROR( __FILE__, __LINE__);
	}
  }


  if(palette_dirty) {
		gl_calibrate_palette();
		palette_dirty=0;
		tx_palette_change=1;
  }

  if(cabview && !cabload_err) UpdateCabDisplay();
  else UpdateFlatDisplay();


  CHECK_GL_ERROR( __FILE__, __LINE__);

  if(vecgame&&inlist) 
  {
	vector_clear_list();
    /* YES - lets catch this error ... 
	   the upper function calls glBegin at the end, tsts */
    glerrid = glGetError();
    if(0x502!=glerrid)
	{
	  CHECK_GL_ERROR( __FILE__, __LINE__);
	}
  }

  screendirty=0;
}
	 
void  gl_set_bilinear(int new_value)
{
	  bilinear=new_value;

	  if(gl_is_initialized==0) return;

	  if(bilinear) {
		glBindTexture(GL_TEXTURE_2D,cabtex[0]);

		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

		glPixelStorei(GL_UNPACK_ROW_LENGTH , memory_x_len);
		glPixelStorei(GL_UNPACK_ALIGNMENT, 8);
		CHECK_GL_ERROR( __FILE__, __LINE__);

		glBindTexture(GL_TEXTURE_2D,texgrid->texobj);
			
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

		glPixelStorei(GL_UNPACK_ROW_LENGTH , 0);
		glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
		CHECK_GL_ERROR( __FILE__, __LINE__);
	  }
	  else {
		glBindTexture(GL_TEXTURE_2D,cabtex[0]);

		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);

		glPixelStorei(GL_UNPACK_ROW_LENGTH , memory_x_len);
		glPixelStorei(GL_UNPACK_ALIGNMENT, 8);
		CHECK_GL_ERROR( __FILE__, __LINE__);

		glBindTexture(GL_TEXTURE_2D,texgrid->texobj);
			
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);

		glPixelStorei(GL_UNPACK_ROW_LENGTH , 0);
		glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
		CHECK_GL_ERROR( __FILE__, __LINE__);
	  }
}

	 
void  gl_set_cabview(int new_value)
{
	  cabview=new_value;

	  if(gl_is_initialized==0) return;

	  if(cabview)
		SetupFrustum();
	  else SetupOrtho();
}

void  gl_set_antialias(int new_value)
{
  antialias=new_value;

  if(gl_is_initialized==0) return;

  if(antialias)
  {
	glShadeModel(GL_SMOOTH);
	glEnable(GL_POLYGON_SMOOTH);
  } else {
	glShadeModel(GL_FLAT);
	glDisable(GL_POLYGON_SMOOTH);
  }
}

void  gl_set_alphablending(int new_value)
{
  alphablending=new_value;

  if(gl_is_initialized==0) return;

  if(alphablending)
  {
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
  } else {
	glDisable(GL_BLEND);
  }
}

/* used when expose events received */
void
#ifdef WIN32
	gl_refresh_screen
#else
	osd_refresh_screen
#endif
	(void)
{
  /* Just re-draw the whole screen */

  if(gl_is_initialized==0) return;
  UpdateGLDisplay();
}


/* invoked by wgl_GDIDisplay.GDI_update_display to update bitmap into screen */

void
#ifdef WIN32
	gl_update_display
#else
	sysdep_update_display
#endif
	(void)
{
  if(gl_is_initialized==0) return;
  UpdateGLDisplay();

  frame++;

#ifndef WIN32
  if(osd_key_pressed(OSD_KEY_F12)) osd_save_snapshot();

  if(osd_key_pressed(OSD_KEY_RCONTROL)) {
	/* if(osd_key_pressed(OSD_KEY_A)) {
	  antialias=1-antialias;

	  while(osd_key_pressed(OSD_KEY_A));
	}
	else   */ if(osd_key_pressed(OSD_KEY_C)&&dodepth) {
	  gl_set_cabview(1-cabview);

	  while(osd_key_pressed(OSD_KEY_C));
	}
	else if(osd_key_pressed(OSD_KEY_B)) {
	  gl_set_bilinear(1-bilinear);

	  while(osd_key_pressed(OSD_KEY_B));
	}
	/* else if(osd_key_pressed(OSD_KEY_O)) {
	  drawbitmap=1-drawbitmap;

	  while(osd_key_pressed(OSD_KEY_O));
	} */
	else if(osd_key_pressed(OSD_KEY_T)) {
	  dopersist=1-dopersist;

	  while(osd_key_pressed(OSD_KEY_T));
	}
  }
#endif
  
}

/*
 * Makes a snapshot of screen when F12 is pressed.
 *
 * The snapshot is stored in ppm format
 * It is saved in user's home dir as gamename-XXX.ppm
 * 
 */
#ifdef WIN32
static char *home_dir = "images";

#include "file.h"
#include "M32Util.h"
#endif

static void do_save_snapshot (void)
{
	#ifdef WIN32
		if(useBMPSnap) 
			bmp_save_snapshot();
	#endif

	if(usePNGSnap) 
		png_save_snapshot();

	if(useJPGSnap) 
		jpg_save_snapshot();

	if(usePPMSnap) 
		ppm_save_snapshot();
}

void 
#ifdef WIN32
	gl_save_snapshot 
#else
	osd_save_snapshot
#endif
	(void)
{
	do_snapshot = 1;
}

void png_save_snapshot (void)
{
    char                sFileName[MAX_PATH];
    int                 nResult;
    void*               pFile;
	int				    snap_no=0;

    /* Create a unique PNG file */
    do
    {
        if (snap_no == 0)    /* first of all try with the "gamename.bmp" */
            sprintf(sFileName, "%.8s", Machine->gamedrv->name);
        else                /* otherwise use "nameNNNN.bmp" */
            sprintf(sFileName, "%.4s%04d", Machine->gamedrv->name, snap_no);

        /* Avoid overwriting of existing files */
        nResult = osd_faccess(sFileName, OSD_FILETYPE_SCREENSHOT);
               
        if (nResult != 0)
            snap_no++;

    } while (nResult != 0);

    if ((pFile = osd_fopen(sFileName, "", OSD_FILETYPE_SCREENSHOT, TRUE)) == NULL)
    {
        ErrorMsg("osd_fopen failed opening %s", sFileName);
        return;
    }

    png_write_bitmap(pFile, bitmap);
 
    osd_fclose(pFile);

}

#ifdef WIN32
void bmp_save_snapshot (void)
{
    BITMAPFILEHEADER    bmfHeader;
    BITMAPINFOHEADER    bmiHeader;
    int                 nWritten;
	int					snap_no=0;

    char						name[1024];
    FILE						*file;

	int pixelsize=0;
	unsigned char *pixels=0;

    int width  = winwidth;
    int height = winheight;

	width+=width%2;   /* having a multiple of 2, WORD alignment rows */
	height+=height%2; /* having a multiple of 2, WORD alignment rows */

    if(gl_is_initialized==0) return;

    /*
        Create a unique BMP file.
    */
    /* avoid overwriting of existing files */
    do {
	  if(snap_no==0)
		sprintf(name,"%s/%.8s.bmp",home_dir,Machine->gamedrv->name,
			    snap_no);
	  else
		sprintf(name,"%s/%.4s%04d.bmp",home_dir,Machine->gamedrv->name,
			snap_no);
	  if((file=fopen(name,"r"))!=0) {
	    fclose(file);
	    snap_no++;
	  }
	}	while(file!=0);
  
  if(!(file=fopen(name,"wb"))) {
	fprintf(stderr,"Failed to save snapshot %s\n",name);
	return ;
  }

    pixelsize = width * height  * 3 /*RGB*/ ;

    bmiHeader.biSize            = sizeof(BITMAPINFOHEADER);
    bmiHeader.biWidth           = width;
    bmiHeader.biHeight          = height;
    bmiHeader.biPlanes          = 1;
    bmiHeader.biBitCount        = 24;
    bmiHeader.biCompression     = BI_RGB;
    bmiHeader.biSizeImage       = pixelsize; 
    bmiHeader.biXPelsPerMeter   = 0;
    bmiHeader.biYPelsPerMeter   = 0;
    bmiHeader.biClrUsed         = 0;
    bmiHeader.biClrImportant    = 0;

    /*
        Init BITMAPFILEHEADER.
    */

    bmfHeader.bfType        = 0x4d42;
    bmfHeader.bfReserved1   = 0;
    bmfHeader.bfReserved2   = 0;
    bmfHeader.bfSize        = sizeof(BITMAPFILEHEADER) +
                              sizeof(BITMAPINFOHEADER) +
                              bmiHeader.biClrUsed * sizeof(RGBQUAD) +
                              bmiHeader.biSizeImage;
    bmfHeader.bfOffBits     = sizeof(BITMAPFILEHEADER) +
                              sizeof(BITMAPINFOHEADER) +
                              bmiHeader.biClrUsed * sizeof(RGBQUAD);

    /* BITMAPFILEHEADER */
    
    nWritten = fwrite((void*)&bmfHeader, sizeof(BITMAPFILEHEADER), 1, file);
    if (1 != nWritten)
        ErrorMsg("osd_fwrite failed writing BITMAPFILEHEADER");
    
    /* BITMAPINFOHEADER */
    
    nWritten = fwrite((void*)&bmiHeader, sizeof(BITMAPINFOHEADER),1, file);
    if (1 != nWritten)
        ErrorMsg("osd_fwrite failed writing BITMAPINFOHEADER");
    
    /* RGBQUAD */
    
    /*
        Write image data line by line, bottom up.
    */

    pixels=(unsigned char *)malloc(pixelsize);

    glPixelStorei(GL_PACK_ALIGNMENT,  4); /* DOUBLE-WORD alignment rows */
    glPixelStorei(GL_PACK_ROW_LENGTH, winwidth);
    glPixelStorei(GL_PACK_LSB_FIRST, GL_TRUE);

	glReadPixels(0,0, winwidth, winheight,
			     GL_RGB, GL_UNSIGNED_BYTE,
			     pixels);

    glPixelStorei(GL_PACK_ALIGNMENT,4);
    glPixelStorei(GL_PACK_ROW_LENGTH, 0);

	filter_rgb_to_bgr(pixels, width, height);

    nWritten = fwrite((void*)pixels, pixelsize, 1, file);

    if (1 != nWritten)
        ErrorMsg("osd_fwrite failed writing image data");

    fclose(file);
}
#endif

void ppm_save_snapshot (void)
{
  char						name[1024];
  FILE						*file;
  int						pixelsize, m, i, pixelnum;
  static unsigned char		*pixels;
  int						snap_no=0;

  int width  = winwidth;
  int height = winheight;

  if(gl_is_initialized==0) return;

  /* avoid overwriting of existing files */
  do {
	  if(snap_no==0)
		sprintf(name,"%s/%.8s.ppm",home_dir,Machine->gamedrv->name,
			    snap_no);
	  else
		sprintf(name,"%s/%.4s%04d.ppm",home_dir,Machine->gamedrv->name,
			snap_no);
	if((file=fopen(name,"r"))!=0) {
	  fclose(file);
	  snap_no++;
	}
  }	while(file!=0);
  
  if(!(file=fopen(name,"wb"))) {
	fprintf(stderr,"Failed to save snapshot %s\n",name);
	return ;
  }
  
  m         = width * 3 /*RGB*/ ;
  pixelsize = width * height  * 3 /*RGB*/ ;

  pixels=(unsigned char *)calloc(pixelsize, 1);

  glPixelStorei(GL_PACK_ALIGNMENT, 1);
  glPixelStorei(GL_PACK_ROW_LENGTH, width);
  glPixelStorei(GL_PACK_LSB_FIRST, GL_TRUE);

  glReadPixels(0,0, width, height,
			   GL_RGB, GL_UNSIGNED_BYTE,
			   pixels);

  glPixelStorei(GL_PACK_ALIGNMENT,4);
  glPixelStorei(GL_PACK_ROW_LENGTH, 0);

  fprintf(file,"P6\n#GLmame screenshot\n%d %d\n255\n",
		  (int)width, (int)height);
  
  
  for(pixelnum=0, i=height-1;i>=0;i--) 
  {
	fwrite(&pixels[i*m], sizeof(unsigned char), m, file);
  }

  fclose(file);
  free(pixels); pixels=0;

  return;
}


void jpg_save_snapshot (void)
{
  JSAMPLE * pixels;	        /* Points to large array of R,G,B-order data */

  int width  = winwidth;
  int height = winheight;
  /*
  int width=visual_width+0.5;
  int height=visual_height+0.5;
  */

  struct jpeg_compress_struct cinfo;
  struct jpeg_error_mgr jerr;

  JSAMPROW row_pointer[1];	/* pointer to JSAMPLE row[s] */
  int row_stride;		/* physical row width in image buffer */

  char						name[1024];
  FILE						*file;
  int						pixelsize, m;
  int						snap_no=0;

  if(gl_is_initialized==0) return;

  cinfo.err = jpeg_std_error(&jerr);
  jpeg_create_compress(&cinfo);

  /* avoid overwriting of existing files */
  do {
	  if(snap_no==0)
		sprintf(name,"%s/%.8s.jpg",home_dir,Machine->gamedrv->name,
			    snap_no);
	  else
		sprintf(name,"%s/%.4s%04d.jpg",home_dir,Machine->gamedrv->name,
			snap_no);
	if((file=fopen(name,"r"))!=0) {
	  fclose(file);
	  snap_no++;
	}
  }	while(file!=0);
  
  if(!(file=fopen(name,"wb"))) {
	fprintf(stderr,"Failed to save snapshot %s\n",name);
	return ;
  }
  
  m         = width * 3 /*RGB*/ ;
  pixelsize = width * height  * 3 /*RGB*/ ;

  pixels=(unsigned char *)calloc(pixelsize, 1);

  glPixelStorei(GL_PACK_ALIGNMENT, 1);
  glPixelStorei(GL_PACK_ROW_LENGTH, width);
  glPixelStorei(GL_PACK_LSB_FIRST, GL_TRUE);

  glReadPixels(0,0, width, height,
			   GL_RGB, GL_UNSIGNED_BYTE,
			   pixels);

  glPixelStorei(GL_PACK_ALIGNMENT,4);
  glPixelStorei(GL_PACK_ROW_LENGTH, 0);

  filter_rgb_to_bgr(pixels, width, height);

  jpeg_stdio_dest(&cinfo, file);

  cinfo.image_width		 = width; 	/* image width and height, in pixels */
  cinfo.image_height	 = height;
  cinfo.input_components = 3;		/* # of color components per pixel */
  cinfo.in_color_space   = JCS_RGB; /* colorspace of input image */

  jpeg_set_defaults(&cinfo);

  jpeg_set_quality(&cinfo, jpgQuality, TRUE /* limit to baseline-JPEG values */);

  jpeg_start_compress(&cinfo, TRUE);

  row_stride = width * 3;	/* JSAMPLEs per row in image_buffer */

  while (cinfo.next_scanline < cinfo.image_height) {
    row_pointer[0] = & pixels[(height-1-cinfo.next_scanline) * row_stride];
    (void) jpeg_write_scanlines(&cinfo, row_pointer, 1);
  }

  jpeg_finish_compress(&cinfo);

  fclose(file);

  jpeg_destroy_compress(&cinfo);


  free(pixels); pixels=0;
  
  return;
}


static void	filter_rgb_to_bgr(unsigned char *pixels, int width, int height)
{
	unsigned char r;
	unsigned char * dest = pixels;
	unsigned char * end  = pixels;

	end+=width*3*height;

	while(dest<end)
	{
		r			= *pixels;		/* save red */
		*dest		= *(pixels+2);	/* place blue to former red */
		*(dest+2)	= r;			/* place red to former blue */

		pixels		+=3;			/* next tripple */
		dest		+=3;
	}
}
 
#endif /* ifdef xgl */
