/***************************************************************************

    M.A.M.E.32  -  Multiple Arcade Machine Emulator for Win32

    Derived from the file "GDIDisplay.c" from
    Win32 Portions Copyright (C) 1997-98 Michael Soderstrom and Chris Kirmse

    This file is part of the OpenGL Version of M.A.M.E., called GLMame32 !
	Copyright 1999 by Sven Goethel, http://www.jausoft.com
    
    This file is part of MAME32, and may only be used, modified and
    distributed under the terms of the MAME license, in "readme.txt".
    By continuing to use, modify or distribute this file you indicate
    that you have read the license and understand and accept it fully.

  many of the generic routines are now implemented in glgen.c
  Modified for GLMame32
  29/April/1999 Sven Goethel

 ***************************************************************************/

/***************************************************************************

  GDIDisplay.c

 ***************************************************************************/

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#include <assert.h>
#include <math.h>
#include <stdio.h>
#include "MAME32.h"
#include "mame.h"
#include "resource.h"
#include "driver.h"
#include "wgl_GDIDisplay.h"
#include "wgl_tool.h"
#include "glmame.h"
#include "M32Util.h"
#include "status.h"
#include "wgl_status.h"
#include "dirty.h"

#define MAKECOL(r, g, b) (((r) & 0x001F) << 10 | ((g) & 0x001F) << 5 | ((b) & 0x001F))

/***************************************************************************
    function prototypes
 ***************************************************************************/

static void             OnCommand(HWND hWnd, int CmdID, HWND hWndCtl, UINT codeNotify);
static void				OnCreate(HWND hWnd);
static void				OnClose(HWND hWnd);
static void             OnPaint(HWND hWnd);
static void             OnSize(HWND hwnd,UINT state,int cx,int cy);
static BOOL             OnEraseBkgnd(HWND hWnd, HDC hDC);
static LRESULT CALLBACK AboutDialogProc(HWND hDlg, UINT Msg, WPARAM wParam, LPARAM lParam);

static int                GDI_init(options_type *options);
static void               GDI_exit(void);
static struct osd_bitmap* GDI_new_bitmap(int width, int height,int depth);
static void               GDI_clearbitmap(struct osd_bitmap* bitmap);
static void               GDI_free_bitmap(struct osd_bitmap* bitmap);
static struct osd_bitmap* GDI_create_display(int width, int height, int depth, int attributes);
static int                GDI_set_display(int width, int height, int attributes);
static void               GDI_close_display(void);
static void               GDI_update_display(void);
static void               GDI_led_w(int led, int on);
static void               GDI_save_snapshot(void);
static BOOL               GDI_OnMessage(HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam, LRESULT* pResult);
static void               GDI_Refresh(void);
static void				  GDI_UpdateFPS(BOOL bShow, int nSpeed, int nFPS, int nMachineFPS, int nFrameskip, int nVecUPS);
static void				  wgl_Display_WriteBitmap(struct osd_bitmap* tBitmap);

/***************************************************************************
    External variables
 ***************************************************************************/

extern int bitmap_dirty;

int game_index;
int winwidth;
int winheight;
int win_left;
int win_bottom;
int visual_width;
int visual_height;


int doublebuffer=1;  /* Do double buffering? */
int dodepth=1;

int use_window = 1;
options_type *save_options_for_gl=0;
float gl_win_scale = 0.0;

struct osd_bitmap* bitmap=0;
struct rectangle	visual;

struct OSDDisplay   wgl_GDIDisplay = 
{
    { GDI_init },               /* init              */
    { GDI_exit },               /* exit              */
    { GDI_new_bitmap },         /* new_bitmap        */
    { GDI_clearbitmap },        /* clearbitmap       */
    { GDI_free_bitmap },        /* free_bitmap       */
    { GDI_create_display },     /* create_display    */
    { GDI_set_display },        /* set_display       */
    { GDI_close_display },      /* close_display     */
    { gl_allocate_colors },     /* allocate_colors   */
    { gl_modify_pen },          /* modify_pen        */
    { gl_get_pen },             /* get_pen           */
    { gl_mark_dirty },          /* mark_dirty        */
    { 0 },						/* skip_this_frame   */
    { GDI_update_display },     /* update_display    */
    { GDI_led_w },              /* led_w             */
    { gl_set_gamma },           /* set_gamma         */
    { gl_get_gamma },			/* get_gamma         */
    { gl_set_brightness },      /* set_brightness    */
    { gl_get_brightness },		/* get_brightness    */
    { GDI_save_snapshot },      /* save_snapshot     */

    { GDI_OnMessage },          /* OnMessage         */
    { GDI_Refresh },            /* Refresh           */
    { gl_GetBlackPen },         /* GetBlackPen       */
    { GDI_UpdateFPS },          /* UpdateFPS         */
};

/***************************************************************************
    Internal structures
 ***************************************************************************/

struct tDisplay_private
{
    RECT                m_ClientRect;
};

/***************************************************************************
    Internal variables
 ***************************************************************************/

static struct tDisplay_private      This;

static unsigned char color_dirty[256];
static const int win_border_delta = 0;

static int set_viewport = 0;
static int gl_ctx_error=0;

FILE * saveErrFile=0;
FILE * saveLogFile=0;

static const int                    safety = 16;

/***************************************************************************
    Internal helper functions  
 ***************************************************************************/

static void _openStdFiles()
{
	if(options.errorlog != 0)
	{
		saveErrFile=freopen ("glmame32_stderr.log", "w", stderr);
		saveLogFile=freopen ("glmame32_stdout.log", "w", stdout);
	}
}

static void _closeStdFiles()
{
	if(saveErrFile)
	{
		fflush(saveErrFile);
		fclose(saveErrFile);
	}
	if(saveLogFile)
	{
		fflush(saveLogFile);
		fclose(saveLogFile);
	}
}

static void _flushStdFiles()
{
	if(options.errorlog != 0)
	{
		fflush(NULL);
	}
}

 /***************************************************************************
    External OSD functions  
 ***************************************************************************/

/*
    put here anything you need to do when the program is started. Return 0 if 
    initialization was successful, nonzero otherwise.
*/
static int GDI_init(options_type *w32_options)
{
	save_options_for_gl=w32_options;

    gamma_correction = max(0, w32_options->gamma);
    gl_alpha_value = (unsigned char)((w32_options->brightness/100.0)*255.0);
    use_window = 1;
	antialias = 0 /* options->antialias */ ; /* better with this display */
	gl_win_scale = w32_options->scale; /* lets use a double value, to float on */

    if(Machine->drv->video_attributes & VIDEO_TYPE_VECTOR)
		alphablending=1;

    glStatusCreate(TRUE);  /* only teh startup tasks .. */

    OSDDisplay.init(w32_options);

	bitmap = NULL;

    return 0;
}

/*
    put here cleanup routines to be executed when the program is terminated.
*/
static BOOL bitmap4GLTexture = FALSE;

static void GDI_exit(void)
{
	bitmap4GLTexture = FALSE;
    OSDDisplay.exit();
}

/* VERY IMPORTANT: the function must allocate also a "safety area" 16 pixels wide all */
/* around the bitmap. This is required because, for performance reasons, some graphic */
/* routines don't clip at boundaries of the bitmap. */
static struct osd_bitmap* GDI_new_bitmap(int width, int height,int depth)
{
	struct osd_bitmap *bitmap;
	unsigned char *bitmap_data=0;

	if (!width) width++;
	if (!height) height++;

	if (Machine->orientation & ORIENTATION_SWAP_XY)
	{
		int temp;

		temp = width;
		width = height;
		height = temp;

		if(bitmap4GLTexture) {
			/* our OpenGL texture dimension must flip either */
			temp		= text_width;
			text_width	= text_height;
			text_height = temp;
		}
	}

	if ((bitmap = malloc(sizeof(struct osd_bitmap))) != 0)
	{
		int i,rowlen,rdwidth, bytes_per_pixel;
		int y_rows;
		int bitmap_size;

		bitmap->depth = depth;
		bitmap->width = width;
		bitmap->height = height;

		rdwidth = (width + 7) & ~7;  /* round width to a quadword */

		bytes_per_pixel=depth/8;

		rowlen = bytes_per_pixel * 
				 (rdwidth + 2 * safety) * sizeof(unsigned char);

		y_rows = height + 2 * safety;

		bitmap_size = y_rows * rowlen ;

		if(bitmap4GLTexture) 
		{
			/* check if we need more ram for our texture buffer,
			   because the dimension is 2**m x 2**n !

			   The rows must not be expanded, just the bitmap
			   memory - because of read access :-)
			 */
			int tw_len, ty_rows, tw;
			
			tw      = (text_width + 7) & ~7; /* round width to a quadword */
			tw_len  = bytes_per_pixel * 
				      (tw + 2 * safety) * sizeof(unsigned char);
			ty_rows = text_height + 2 * safety ;
			
			bitmap_size = ty_rows * tw_len;

			bitmap4GLTexture = FALSE; /* thats it */

			if ((bitmap_data = malloc(bitmap_size)) == 0)
			{
				free(bitmap);
				return 0;
			}
		} else {
			if ((bitmap_data = malloc(bitmap_size)) == 0)
			{
				free(bitmap);
				return 0;
			}
		}

        /* clear ALL bitmap, including safety area, to avoid garbage on right */
        /* side of screen is width is not a multiple of 4 */
        memset(bitmap_data, 0, (height + 2 * safety) * rowlen);

        if ((bitmap->line = malloc((height + 2 * safety) * sizeof(unsigned char *))) == 0)
        {
            free(bitmap_data);
            free(bitmap);
            return 0;
        }

        for (i = 0; i < height + 2 * safety; i++)
        {
			bitmap->line[i] = &bitmap_data[i * rowlen + safety * bytes_per_pixel];
        }

        bitmap->line += safety;

        bitmap->_private = bitmap_data ;

        osd_clearbitmap(bitmap);
	}

	return bitmap;
}

static void GDI_free_bitmap(struct osd_bitmap* bitmap)
{
    if (bitmap != NULL)
    {
        bitmap->line -= safety;
        free(bitmap->line);
        free(bitmap->_private);
        free(bitmap);
    }
}

static void GDI_clearbitmap(struct osd_bitmap* bitmap)
{
	int i;

    for (i = 0; i < bitmap->height; i++)
        memset(bitmap->line[i], 0, bitmap->width * (bitmap->depth / 8));

	if (bitmap == Machine->scrbitmap)
	{
		bitmap_dirty = 1;
		osd_mark_dirty (0,0,bitmap->width-1,bitmap->height-1,1);
	}


}


/*
    Create a display screen, or window, large enough to accomodate a bitmap
    of the given dimensions.
    Return a osd_bitmap pointer or 0 in case of error.
*/
static struct osd_bitmap *GDI_create_display(int width, int height, int depth, int attributes)
{
	RECT			Rect;
    HMENU           hMenu;
    char            TitleBuf[256];
	int a;
	int w,h;

   if (Machine->drv->video_attributes & VIDEO_TYPE_VECTOR)
   {
	  /*
      width  = width  * gl_win_scale + 0.5;
      height = height * gl_win_scale + 0.5;
	  */

      /* round with & heigth to a multiple of 8 */
      width  &= ~7;
      height &= ~7;

      /* store info into a visual struct */
      visual.min_x = 0;
      visual.min_y = 0;
      visual.max_x = width - 1;
      visual.max_y = height - 1;
   } else {
      visual = Machine->drv->visible_area;
   }

   /* round depth to a multiple of 8 */
   depth  &= ~7;

   /* yes create a OpenGL-Texture compatible bitmap this time ... */
   bitmap4GLTexture=TRUE; 

   /* JAU: find some n,m for 
		(w=2**n)>=text_width, (h=2**m)>=text_height */
   /* we try to use the bitmap for OpenGL textures directly 
		.. no copies */
 	text_width  = width;
	text_height = height;
	w=1; h=1;

	while (w<text_width)
	{ w*=2; }

	while (h<text_height)
	{ h*=2; }

	text_width=w;
	text_height=h;

   bitmap = osd_new_bitmap(width, height, depth);
   Machine->scrbitmap = bitmap;

   if (bitmap==NULL) return NULL;
   
   if( Machine->orientation & ORIENTATION_SWAP_XY) {
      a=visual.max_x; visual.max_x=visual.max_y; visual.max_y=a;
      a=visual.min_x; visual.min_x=visual.min_y; visual.min_y=a;
      a=width; width=height; height=a;
   } 

   if (Machine->orientation & ORIENTATION_FLIP_X)
   {
      a            = width - visual.min_x - 1;
      visual.min_x = width - visual.max_x - 1;
      visual.max_x = a;
   }
   
   if (Machine->orientation & ORIENTATION_FLIP_Y)
   {
      a            = height - visual.min_y - 1;
      visual.min_y = height - visual.max_y - 1;
      visual.max_y = a;
   }
     
   /* round to 8, since the new dirty code works with 8x8 blocks,
      and we need to round to sizeof(long) for the long copies anyway */
   if (visual.min_x & 7)
   {
      if((visual.min_x - (visual.min_x & ~7)) < 4)
         visual.min_x &= ~7;
       else
         visual.min_x = (visual.min_x + 7) & ~7;
   }
   if ((visual.max_x+1) & 7)
   {
      if(((visual.max_x+1) - ((visual.max_x+1) & ~7)) > 4)
         visual.max_x = ((visual.max_x+1 + 7) & ~7) - 1;
       else
         visual.max_x = ((visual.max_x+1) & ~7) - 1;
   }
   /* now calculate the visual width / height */
   visual_width  = visual.max_x - visual.min_x + 1;
   visual_height = visual.max_y - visual.min_y + 1;

   winwidth= visual_width   + 2*win_border_delta ;
   winheight= visual_height + 2*win_border_delta ;

   win_left   =win_border_delta;
   win_bottom =win_border_delta;

	winwidth				= winwidth				* gl_win_scale + 0.5;
	winheight				= winheight				* gl_win_scale + 0.5;
	win_left				= win_left				* gl_win_scale + 0.5;
	win_bottom				= win_bottom			* gl_win_scale + 0.5;

    hMenu=0;

    sprintf(TitleBuf, "%s - %s", Machine->gamedrv->description, MAME32App.m_Name);
    SetWindowText(MAME32App.m_hWnd, TitleBuf);
	
	This.m_ClientRect.left   = 0;
	This.m_ClientRect.top    = 0;
	This.m_ClientRect.right  = winwidth - 1;
	This.m_ClientRect.bottom = winheight - 1;

	if(use_window)
	{
		/* Calculate size of window based on desired client area. */
		Rect.left   = This.m_ClientRect.left;
		Rect.top    = This.m_ClientRect.top;
		Rect.right  = This.m_ClientRect.right;
		Rect.bottom = This.m_ClientRect.bottom;
		AdjustWindowRect(&Rect, WS_OVERLAPPEDWINDOW & ~WS_THICKFRAME | WS_BORDER, hMenu != NULL);

		SetWindowPos(MAME32App.m_hWnd,
                 HWND_NOTOPMOST,
                 0, 0,
				 RECT_WIDTH(Rect),
				 RECT_HEIGHT(Rect),
                 SWP_NOMOVE);
	} else {
		/*
			Modify the main window to suit our needs.
		*/
		SetWindowLong(MAME32App.m_hWnd, GWL_STYLE, WS_POPUP);
		SetWindowPos(MAME32App.m_hWnd,
                 HWND_TOPMOST,
                 0, 0,
                 GetSystemMetrics(SM_CXSCREEN) /* /4 */,
                 GetSystemMetrics(SM_CYSCREEN) /* /4 */,
                 0);

		ShowWindow(MAME32App.m_hWnd, SW_SHOW);
	}

    ShowWindow(MAME32App.m_hWnd, SW_SHOW);
    SetForegroundWindow(MAME32App.m_hWnd);

	OnCreate(MAME32App.m_hWnd);
	
	if(glContext==0) return 0;

	InitVScreen();

   set_ui_visarea (visual.min_x, visual.min_y, visual.max_x, visual.max_y);
   
   if(use_window)
		glStatusCreate(FALSE); /* creates Status also .. */

   UpdateWindow(MAME32App.m_hWnd);

   return bitmap;
}

/*
    Set the actual display screen but don't allocate the screen bitmap.
*/
static int GDI_set_display(int width, int height, int attributes)
{
    return 0; 
}

/*
     Shut down the display
*/
static void GDI_close_display(void)
{
	/*
    ExitDirty();
	*/
    
    glStatusDelete(); /* deletes Status also .. */

	OnClose(MAME32App.m_hWnd);
}


static void GDI_update_display(void)
{
    InvalidateRect(MAME32App.m_hWnd, &This.m_ClientRect, FALSE);

	/* the opengl update will be made in the event handle OnPaint() */
    UpdateWindow(MAME32App.m_hWnd);

    glStatusUpdate();

    MAME32App.ProcessMessages();
}

/* control keyboard leds or other indicators */
static void GDI_led_w(int led, int on)
{
    glStatusWrite(led, on & 1);
}

static void GDI_save_snapshot()
{
	gl_save_snapshot ();
}

void wgl_Display_WriteBitmap(struct osd_bitmap* tBitmap)
{
}


/***************************************************************************
    Internal functions
 ***************************************************************************/

static BOOL GDI_OnMessage(HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam, LRESULT* pResult)
{
    switch (Msg)
    {
        HANDLE_MESSAGE(hWnd, WM_COMMAND,            OnCommand);
		HANDLE_MESSAGE(hWnd, WM_PAINT,              OnPaint);
        HANDLE_MESSAGE(hWnd, WM_ERASEBKGND,         OnEraseBkgnd);
        HANDLE_MESSAGE(hWnd, WM_SIZE,               OnSize);
    }
    return FALSE;
}

static void GDI_Refresh()
{
    InvalidateRect(MAME32App.m_hWnd, &This.m_ClientRect, FALSE); 
	/* gl_refresh_screen(); */
    UpdateWindow(MAME32App.m_hWnd); 
}

static void GDI_UpdateFPS(BOOL bShow, int nSpeed, int nFPS, int nMachineFPS, int nFrameskip, int nVecUPS)
{
    StatusUpdateFPS(bShow, nSpeed, nFPS, nMachineFPS, nFrameskip, nVecUPS);
}

static void OnCommand(HWND hWnd, int CmdID, HWND hWndCtl, UINT codeNotify)
{
    switch (CmdID)
    {
        case ID_FILE_EXIT:
            /* same as double-clicking on main window close box */
            SendMessage(hWnd, WM_CLOSE, 0, 0);
        break;
    }
}

static void OnCreate(HWND hWnd)
{
    HDC		hDC;
	HGLRC glCx = 0;
	int pixFmt;

	_openStdFiles();

	if(glContext==0 && !gl_ctx_error)
	{
        hDC = GetDC(hWnd);

		if(alphablending)
		{
			glCx = wglToolGetGLContext( hWnd, hDC, 32 /*RGBA*/, 16,
										(options.errorlog==0)?0:1,
									    &doublebuffer, &pixFmt);
		} else {
			glCx = wglToolGetGLContext( hWnd, hDC, 24 /*RGB*/, 16,
										(options.errorlog==0)?0:1,
									    &doublebuffer, &pixFmt);
		}

		if(glCx==0) 
		{
			gl_ctx_error=1;
			fprintf(stderr,"GL Up-Error !\n");
		} 
		else fprintf(stderr,"GL Up !\n");
	}
}

static void OnClose(HWND hWnd)
{
    HGLRC hRC;
    HDC	  hDC;

	if( hWnd !=MAME32App.m_hWnd ) return;

    /* release and free the device context and rendering context */
    hRC = wglGetCurrentContext();
    hDC = wglGetCurrentDC();

    if (glContext!=hRC)
	{
		fprintf(stderr,"who has changed the glContext ?\n");
	}
	CloseVScreen();
	glContext=0;

    if (glHDC!=hDC)
	{
		fprintf(stderr,"who has changed the glHDC ?\n");
	}
	glHDC=0;

    wglMakeCurrent(NULL, NULL);

	/* THIS MAKES AN EXCEPTIONS */
    if (hRC)
    	wglDeleteContext(hRC);

    if (hDC)
        ReleaseDC(hWnd, hDC);

    osd_free_bitmap(bitmap);
	bitmap = NULL;
	fprintf(stderr,"GL Down !\n");

	_closeStdFiles();
}

void OnPaint(HWND hWnd)
{
    HDC		hDC;
    PAINTSTRUCT	ps; 
	static int paintingNum = 0;

	if(gl_is_initialized  && hWnd==MAME32App.m_hWnd && paintingNum<1)
	{
		paintingNum++;
		hDC = BeginPaint(hWnd, &ps);

		glHDC=hDC;

		if(set_viewport)
		{
			glViewport(0,0,winwidth,winheight);
			set_viewport = 0;
		}
		gl_update_display();

	    EndPaint(hWnd, &ps); 

	    _flushStdFiles();
		paintingNum--;
	}
}


static BOOL OnEraseBkgnd(HWND hWnd, HDC hDC)
{
    /* No background erasing is required for OpenGL. */
    return TRUE;
}

static void OnSize(HWND hwnd,UINT state,int cx,int cy)
{
	winwidth=cx;
	winheight=cy;
	set_viewport=1;
}

static LRESULT CALLBACK AboutDialogProc(HWND hDlg, UINT Msg, WPARAM wParam, LPARAM lParam)
{
    switch (Msg)
    {
        case WM_INITDIALOG:
            return 1;

        case WM_COMMAND:
            ShowWindow(hDlg, SW_HIDE);
            return 1;
    }
    return 0;
}

