/***************************************************************************

    M.A.M.E.32  -  Multiple Arcade Machine Emulator for Win32

    Derived from the file "status.c" from
    Win32 Portions Copyright (C) 1997-98 Michael Soderstrom and Chris Kirmse

    This file is part of the OpenGL Version of M.A.M.E., called GLMame32 !
	Copyright 1999 by Sven Goethel, http://www.jausoft.com
    
    This file is part of MAME32, and may only be used, modified and
    distributed under the terms of the MAME license, in "readme.txt".
    By continuing to use, modify or distribute this file you indicate
    that you have read the license and understand and accept it fully.

 ***************************************************************************/

 /***************************************************************************

  status.c

  Deals with maintaining a status bar control on the bottom of a window

  Now - this is an own dialoge, which can be also turned off !
  For OpenGL - many "online" features can be set !

  Modified for GLMame32
  29/April/1999 Sven Goethel


***************************************************************************/

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#include <assert.h>
#include <math.h>
#include <resource.h>
#include <stdio.h>
#include <malloc.h>
#include "commctrl.h"
#include "mame32.h"
#include "m32util.h"
#include "display.h"
#include "mame.h"
#include "driver.h"
#include "uclock.h"
#include "wgl_status.h"
#include "wgl_GDIDisplay.h"
#include "wgl_tool.h"
#include "glmame.h"
#include "status.h"

extern int frameskip;

/***************************************************************************
    Internal function prototypes
 ***************************************************************************/

static void InitializeMisc(HWND hDlg, BOOL setCtrl);
static LRESULT CALLBACK OpenGLStateProc(HWND hDlg, UINT Msg, WPARAM wParam, LPARAM lParam);
static void OptOnHScroll(HWND hwnd, HWND hwndCtl, UINT code, int pos);
static void OnDrawItem(HWND hwnd, const DRAWITEMSTRUCT *lpDrawItem);


/***************************************************************************
    Internal structures
 ***************************************************************************/

/***************************************************************************
    Internal variables
 ***************************************************************************/

static HWND hDlgGLState;
static DWORD prev_update_time;
static int zoom_freescale= 0;
static float zoom_scale = 2;
static float zoom_max = 4.0;

/***************************************************************************
    External functions  
 ***************************************************************************/

void glStatusCreate(BOOL modal)
{
  HWND helpWnd=0;

  prev_update_time = 0;

  if(modal==TRUE)
  {
    if(Machine->drv->video_attributes & VIDEO_TYPE_VECTOR)
	{
		alphablending=1;
		Button_Enable(GetDlgItem(hDlgGLState, IDC_GL_ALPHABLENDING), FALSE);
	}

	DialogBox(GetModuleHandle(NULL), 
			  MAKEINTRESOURCE(IDD_GL_STATUS), NULL /* MAME32App.m_hWnd */,
			  OpenGLStateProc);

  } else {
	hDlgGLState = CreateDialog(GetModuleHandle(NULL),
						   MAKEINTRESOURCE(IDD_GL_STATUS), NULL /*MAME32App.m_hWnd*/,
						   OpenGLStateProc);

	SetWindowPos(hDlgGLState, HWND_BOTTOM /* after all */,
		         0, 0, /* no move */
				 0, 0, /* no size */
				 /* SWP_NOACTIVATE | */
				 SWP_NOMOVE | SWP_NOSIZE );

	/* already started */
	Button_SetText(GetDlgItem(hDlgGLState, IDC_GL_START), "Close");
	ComboBox_Enable(GetDlgItem(hDlgGLState, IDC_GL_CABINETS), FALSE);
	Button_Enable(GetDlgItem(hDlgGLState, IDC_GL_FULLSCREEN), FALSE);
    Button_Enable(GetDlgItem(hDlgGLState, IDC_GL_AUTOSKIP), FALSE);
    Button_Enable(GetDlgItem(hDlgGLState, IDC_GL_FRAMESKIP), FALSE);
	Button_Enable(GetDlgItem(hDlgGLState, IDC_GL_USEEXT), FALSE);
	Button_Enable(GetDlgItem(hDlgGLState, IDC_GL_ALPHABLENDING), FALSE);
	Edit_SetReadOnly(GetDlgItem(hDlgGLState, IDC_GL_FORCE_TEXTWH), TRUE);

	/* force status to be for us ... */
	helpWnd = MAME32App.m_hWnd;
	MAME32App.m_hWnd = hDlgGLState;
    StatusCreate();
	MAME32App.m_hWnd = helpWnd;

    StatusWrite(0, 0);
    StatusWrite(1, 0);
    StatusWrite(2, 0);

	glStatusSetString("F11 - fps view");
    InvalidateRect(hDlgGLState, NULL, TRUE);
	UpdateWindow(hDlgGLState);
  }

}

void glStatusDelete()
{
	if(hDlgGLState==0) return;
    DestroyWindow(hDlgGLState);
	hDlgGLState=0;
	StatusDelete();
}

void glStatusUpdate()
{
	if(hDlgGLState==0) return;
    StatusUpdate();
}

int glGetStatusHeight()
{
	return 0;
}

void glStatusWindowSize(UINT state,int cx,int cy)
{
	(void)state; (void)cx; (void)cy;
}

void glStatusDrawItem(const DRAWITEMSTRUCT *lpdi)
{
	if(hDlgGLState==0) return;
	StatusDrawItem(lpdi);
}

void glStatusWrite(int led,BOOL on)
{
	if(hDlgGLState==0) return;
	StatusWrite(led, on);
}

void glStatusSetString(char *str)
{
	if(hDlgGLState==0) return;
	StatusSetString(str);
}

/* Moved here because it is called in several places */
static void InitializeMisc(HWND hDlg, BOOL setCtrl)
{
	char buf[100];

	if(hDlgGLState==0) return;

	#ifdef GL_EXT_paletted_texture
      if(glColorTableEXT!=0)
			Static_SetText(GetDlgItem(hDlgGLState, IDC_GL_EXTDISP), "ext ok");
	  else
	#endif
			Static_SetText(GetDlgItem(hDlgGLState, IDC_GL_EXTDISP), "no ext");

	if(setCtrl)
	{

	  ComboBox_ResetContent(GetDlgItem(hDlg, IDC_GL_CABINETS));
	  ComboBox_AddString(GetDlgItem(hDlg, IDC_GL_CABINETS), "glmame");
	  ComboBox_AddString(GetDlgItem(hDlg, IDC_GL_CABINETS), "trans");
	  ComboBox_SetCurSel(GetDlgItem(hDlg, IDC_GL_CABINETS), 0);
      Button_SetCheck(GetDlgItem(hDlg, IDC_GL_CABVIEW), cabview);
      Button_SetCheck(GetDlgItem(hDlg, IDC_GL_DOUBLEBUFFER), doublebuffer);
      Button_SetCheck(GetDlgItem(hDlg, IDC_GL_ANTIALIAS), antialias);
      Button_SetCheck(GetDlgItem(hDlg, IDC_GL_BILINEAR), bilinear);
      Button_SetCheck(GetDlgItem(hDlg, IDC_GL_FULLSCREEN), 1-use_window);
      Button_SetCheck(GetDlgItem(hDlg, IDC_GL_ALPHABLENDING), alphablending);
      Button_SetCheck(GetDlgItem(hDlg, IDC_GL_AUTOSKIP), save_options_for_gl->auto_frame_skip);
      Button_SetCheck(GetDlgItem(hDlg, IDC_GL_USEEXT), glGetUseEXT());
      Button_SetCheck(GetDlgItem(hDlg, IDC_GL_SNAPBMP), useBMPSnap);
      Button_SetCheck(GetDlgItem(hDlg, IDC_GL_SNAPJPG), useJPGSnap);
      Button_SetCheck(GetDlgItem(hDlg, IDC_GL_SNAPPNG), usePNGSnap);
      Button_SetCheck(GetDlgItem(hDlg, IDC_GL_SNAPPPM), usePPMSnap);
	  if(!glHasEXT())
		Button_Enable(GetDlgItem(hDlgGLState, IDC_GL_USEEXT), FALSE);

      if(!useJPGSnap)
		Button_Enable(GetDlgItem(hDlgGLState, IDC_GL_JPGQUAL), FALSE);
	  else
		Button_Enable(GetDlgItem(hDlgGLState, IDC_GL_JPGQUAL), TRUE);
	}

	if(setCtrl)
	{
      SendMessage(GetDlgItem(hDlg, IDC_GL_JPGQUAL), TBM_SETRANGE,
				(WPARAM) TRUE,
				(LPARAM) MAKELONG(1, 100));

      SendMessage(GetDlgItem(hDlg, IDC_GL_JPGQUAL), TBM_SETPOS,
				(WPARAM) TRUE,
                (LPARAM) (jpgQuality));
    }
    sprintf(buf, "%3d%%", jpgQuality);
    Static_SetText(GetDlgItem(hDlg, IDC_GL_JPGQUALDISP), buf);

	if(setCtrl)
	{
      SendMessage(GetDlgItem(hDlg, IDC_GL_GAMMA), TBM_SETRANGE,
				(WPARAM) TRUE,
				(LPARAM) MAKELONG(0, 20));

      SendMessage(GetDlgItem(hDlg, IDC_GL_GAMMA), TBM_SETPOS,
				(WPARAM) TRUE,
                (LPARAM) (gamma_correction*10));
    }
    sprintf(buf, "%2.2f", gamma_correction);
    Static_SetText(GetDlgItem(hDlg, IDC_GL_GAMMADISP), buf);

    sprintf(buf, "%d", force_text_width_height);
	Edit_SetText(GetDlgItem(hDlg, IDC_GL_FORCE_TEXTWH), buf);


	if(setCtrl)
	{
      SendMessage(GetDlgItem(hDlg, IDC_GL_ALPHA), TBM_SETRANGE,
				(WPARAM) TRUE,
				(LPARAM) MAKELONG(0, 255));

      SendMessage(GetDlgItem(hDlg, IDC_GL_ALPHA), TBM_SETPOS,
				(WPARAM) TRUE,
                (LPARAM) gl_get_alpha());
    }
    sprintf(buf, "%3d", gl_get_alpha());
    Static_SetText(GetDlgItem(hDlg, IDC_GL_ALPHADISP), buf);

	if(setCtrl)
	{
      SendMessage(GetDlgItem(hDlg, IDC_GL_FRAMESKIP), TBM_SETRANGE,
				(WPARAM) TRUE,
				(LPARAM) MAKELONG(0, 4));

      SendMessage(GetDlgItem(hDlg, IDC_GL_FRAMESKIP), TBM_SETPOS,
				(WPARAM) TRUE,
				(LPARAM) save_options_for_gl->frame_skip);
    
      SendMessage(GetDlgItem(hDlg, IDC_GL_FRAMESKIP), TBM_SETTICFREQ, 
			    (WPARAM) 1,
                (LPARAM) 0);
    }
    sprintf(buf, "%1d", save_options_for_gl->frame_skip);
    Static_SetText(GetDlgItem(hDlg, IDC_GL_FRAMESKIPDISP), buf);

	if(save_options_for_gl->auto_frame_skip)
		Button_Enable(GetDlgItem(hDlgGLState, IDC_GL_FRAMESKIP), FALSE);
	else
		Button_Enable(GetDlgItem(hDlgGLState, IDC_GL_FRAMESKIP), TRUE);

	if(setCtrl)
	{
		SendMessage(GetDlgItem(hDlg, IDC_GL_ZOOM), TBM_SETRANGE,
				(WPARAM) TRUE,
				(LPARAM) MAKELONG(0, zoom_scale*zoom_max));

		SendMessage(GetDlgItem(hDlg, IDC_GL_ZOOM), TBM_SETPOS,
				(WPARAM) TRUE,
				(LPARAM) gl_win_scale * zoom_scale);
    
		SendMessage(GetDlgItem(hDlg, IDC_GL_ZOOM), TBM_SETTICFREQ, 
			    (WPARAM) 1,
                (LPARAM) 0);
	}
	sprintf(buf, "%2.1f", gl_win_scale);
    Static_SetText(GetDlgItem(hDlg, IDC_GL_ZOOMDISP), buf);
}


static LRESULT CALLBACK OpenGLStateProc(HWND hDlg, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	int value;
	char buffer[255];

    if (Msg == WM_INITDIALOG)
	{
		if(hDlgGLState == 0)
			hDlgGLState = hDlg;
		InitializeMisc(hDlg, TRUE);
        return 1;
	}

	if(hDlgGLState==0) return 0;

	switch (Msg)
    {
      case WM_QUIT:
		  if(hDlgGLState!=0)
		  {
			EndDialog( hDlgGLState, TRUE);
			glStatusDelete();
          }
		  return 1;

	  case WM_DRAWITEM:
   	    OnDrawItem(hDlg, (const DRAWITEMSTRUCT *)lParam);
        return 1;

      case WM_HSCROLL:
        /* slider changed */
		/* Get the current value of the control */
          HANDLE_WM_HSCROLL(hDlg, wParam, lParam, OptOnHScroll);
          return 1;

      case WM_COMMAND:
      {
            /* Below, 'changed' is used to signify the 'Apply'
             * button should be enabled.
             */
            WORD wID         = GET_WM_COMMAND_ID(wParam, lParam);
            HWND hWndCtrl    = GET_WM_COMMAND_HWND(wParam, lParam);
            WORD wNotifyCode = GET_WM_COMMAND_CMD(wParam, lParam);

			if(hDlg != hDlgGLState) return FALSE;

            switch (wID)
            {
			case IDC_GL_START:
				EndDialog( hDlgGLState, TRUE);
				glStatusDelete();
				return 1;

            case IDC_GL_CABVIEW:
				gl_set_cabview(1-cabview);
				return 1;

            case IDC_GL_DOUBLEBUFFER:
				doublebuffer=1-doublebuffer;
				return 1;

            case IDC_GL_ALPHABLENDING:
				gl_set_alphablending(1-alphablending);
				return 1;

            case IDC_GL_ANTIALIAS:
				gl_set_antialias(1-antialias);
				return 1;

            case IDC_GL_BILINEAR:
				gl_set_bilinear(1-bilinear);
				return 1;

			case IDC_GL_USEEXT:
				value=1-glGetUseEXT();
				glSetUseEXT(value);
				if(value!=glGetUseEXT())
				{
					Button_Enable(GetDlgItem(hDlgGLState, IDC_GL_USEEXT), FALSE);
					Button_SetCheck(GetDlgItem(hDlg, IDC_GL_USEEXT), glGetUseEXT());
				}
				return 1;

			case IDC_GL_AUTOSKIP:
				save_options_for_gl->auto_frame_skip = 
					1-save_options_for_gl->auto_frame_skip;
				InitializeMisc(hDlg, FALSE);
				return 1;

            case IDC_GL_FULLSCREEN:
				use_window=1-use_window;
				save_options_for_gl->is_window=use_window;
				return 1;

            case IDC_GL_FREESCALE:
				zoom_freescale=1-zoom_freescale;
				if(zoom_freescale)
					zoom_scale=10;
				else
					zoom_scale=2;
				InitializeMisc(hDlg, TRUE);
				return 1;

            case IDC_GL_CABINETS:
				value=ComboBox_GetCurSel(GetDlgItem(hDlg, IDC_GL_CABINETS));
				ComboBox_GetText(GetDlgItem(hDlg, IDC_GL_CABINETS), cabname, 255);
				cabspecified=1;
				return 1;

			case IDC_GL_SNAPBMP:
				useBMPSnap = 1-useBMPSnap;
				return 1;

			case IDC_GL_SNAPJPG:
				useJPGSnap = 1-useJPGSnap;
				if(!useJPGSnap)
					Button_Enable(GetDlgItem(hDlgGLState, IDC_GL_JPGQUAL), FALSE);
				else
					Button_Enable(GetDlgItem(hDlgGLState, IDC_GL_JPGQUAL), TRUE);
				return 1;

			case IDC_GL_SNAPPNG:
				usePNGSnap = 1-usePNGSnap;
				return 1;

			case IDC_GL_SNAPPPM:
				usePPMSnap = 1-usePPMSnap;
				return 1;

			case IDC_GL_FORCE_TEXTWH:
				Edit_GetText(GetDlgItem(hDlgGLState, IDC_GL_FORCE_TEXTWH), buffer, 254);
				force_text_width_height=atoi(buffer);
				return 1;

			default:
				return 0;
			}
      }
    }
    return 0;
}

static void OptOnHScroll(HWND hwnd, HWND hwndCtl, UINT code, int pos)
{
   int value; float valuef;

   if(hDlgGLState==0) return;

   if (hwndCtl == GetDlgItem(hwnd, IDC_GL_GAMMA))
   {
		valuef = SendMessage(GetDlgItem(hwnd, IDC_GL_GAMMA), TBM_GETPOS, 0, 0);
		valuef = (float)valuef/10.0;
		osd_set_gamma(valuef);
		save_options_for_gl->gamma=valuef; 
		InitializeMisc(hwnd, FALSE);
   }
   else if (hwndCtl == GetDlgItem(hwnd, IDC_GL_ALPHA))
   {
		value = SendMessage(GetDlgItem(hwnd, IDC_GL_ALPHA), TBM_GETPOS, 0, 0);
		gl_set_alpha(value);
		save_options_for_gl->gamma=value; 
		InitializeMisc(hwnd, FALSE);
   }
   else if (hwndCtl == GetDlgItem(hwnd, IDC_GL_FRAMESKIP))
   {
		frameskip = 
			SendMessage(GetDlgItem(hwnd, IDC_GL_FRAMESKIP), TBM_GETPOS, 0, 0);
		save_options_for_gl->frame_skip = frameskip;
		InitializeMisc(hwnd, FALSE);
   }
   else if (hwndCtl == GetDlgItem(hwnd, IDC_GL_ZOOM))
   {
		gl_win_scale=
			SendMessage(GetDlgItem(hwnd, IDC_GL_ZOOM), TBM_GETPOS, 0, 0);
		gl_win_scale /= zoom_scale;

		InitializeMisc(hwnd, FALSE);

		if(glWnd!=0) 
		{
			winwidth	= visual_width  * gl_win_scale + 0.5;
			winheight	= visual_height * gl_win_scale + 0.5;

			SetWindowPos(glWnd, hDlgGLState /* after me */,
				         0, 0, /* no move */
						 winwidth, winheight,
						 /* SWP_NOACTIVATE | */
						 SWP_NOMOVE | SWP_SHOWWINDOW );
		}
   }
   else if (hwndCtl == GetDlgItem(hwnd, IDC_GL_JPGQUAL))
   {
		jpgQuality = 
			SendMessage(GetDlgItem(hwnd, IDC_GL_JPGQUAL), TBM_GETPOS, 0, 0);
		InitializeMisc(hwnd, FALSE);
   }
}

static void OnDrawItem(HWND hwnd, const DRAWITEMSTRUCT *lpDrawItem)
{
	if(hDlgGLState==0) return;
    StatusDrawItem(lpDrawItem);
}
