/*****************************************************************

  This file is part of the OpenGL Version of M.A.M.E., called GLMame32 !
  Copyright 1999 by Sven Goethel, http://www.jausoft.com

  This code may be used and distributed under the terms of the
  Mame license

  Modified for GLMame32
  29/April/1999 Sven Goethel

*****************************************************************/
/* pretend we're x11.c otherwise display and a few other crucial things don't
   get declared */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "wgl_tool.h"

HDC glHDC=0;
HWND glWnd=0;
HGLRC glContext=0;

    static PIXELFORMATDESCRIPTOR pfd = {
	    sizeof(PIXELFORMATDESCRIPTOR),  // Size of this structure
	    1,                              // Version of this structure
	    PFD_DRAW_TO_WINDOW              // Draw to Window (not to bitmap)
	    | PFD_SUPPORT_OPENGL	        // Support OpenGL calls in window
	    /* | PFD_DOUBLEBUFFER */,       // Double buffered is optional
	    PFD_TYPE_RGBA,                  // RGBA Color mode
	    24,                             // Want 24bit color
	    0,0,0,0,0,0,                    // Not used to select mode
	    0,0,                            // Not used to select mode
	    0,0,0,0,0,                      // Not used to select mode
	    32,                             // Size of depth buffer
	    0,                              // Not used to select mode
	    0,                              // Not used to select mode
	    PFD_MAIN_PLANE,				    // Draw in main plane
	    0,                              // Not used to select mode
	    0,0,0 };                        // Not used to select mode


/***************************************************************************
    Internal OPENGL functions
 ***************************************************************************/

// Color Palette handle
static HPALETTE hPalette = NULL;

// Set Pixel Format function - forward declaration
static int SetDCPixelFormat(HDC hDC, int dbuffer);

static HPALETTE GetOpenGLPalette(HDC hDC);

static const char * GetTextualPixelFormatByHDC(HDC hdc);
static const char * GetTextualPixelFormatByPFD(PIXELFORMATDESCRIPTOR *ppfd, int format);


static void showLastError()
{
	LPVOID lpMsgBuf;
	
	FormatMessage( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER | \
		FORMAT_MESSAGE_FROM_SYSTEM,    NULL,
		GetLastError(),
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMsgBuf,    0,    NULL );// Display the string.

	MessageBox(NULL, lpMsgBuf, "GL-Context-Error", 
			   MB_OK|MB_ICONERROR); 

	// Free the buffer.
	LocalFree( lpMsgBuf );
}

/* To select the pixel format -> We use a BRUTE FORCE ;-) */
/* We check from :
		color-planes		depth-planes		doublebuffer
		------------------------------------------------------
		   24					32					1
		   24					 0					1
		   
		   24					XX					1
		    8					XX					1
		
		   24					32					0
		   24					 0					0
			   
		   24					XX					0
		    8					XX					0
*/

static const char * nav = "not avaiable";

HGLRC wglToolGetGLContext( HWND win, HDC hdc, 
						   int colorPlanes, int depthPlanes,
						   int verbose,
						   int * pDoublebuffer, 
						   int *pixFmtIdx)
{
	int pixFmt=0;
	const char * text=0;
	HGLRC tempRC=0;


	*pixFmtIdx=0;
	glContext= 0;
	glWnd = win;
	glHDC = hdc;

	if(glHDC==0)
		glHDC = GetDC(glWnd);

	if( glWnd == 0 || glHDC == 0)
		return 0; 

    wglMakeCurrent(NULL, NULL);

	if(colorPlanes<=0) colorPlanes = 24; /* should be enough ;-) */
	if(depthPlanes<=0) depthPlanes = 32;

	while(tempRC==0 && colorPlanes>=8)
	{
		pfd.cColorBits = colorPlanes ;
		pfd.cDepthBits = depthPlanes ;

		if((pixFmt=SetDCPixelFormat(glHDC, *pDoublebuffer)))
		{
			// Create palette if needed
			hPalette = GetOpenGLPalette(glHDC);

			if(verbose)
			{
				fprintf(stderr,"\n\nTry GL-Context for PIXELFORMAT (cp=%d, dp=%d, db=%d):\n",
						colorPlanes, depthPlanes, *pDoublebuffer
					   );
				text=GetTextualPixelFormatByPFD(&pfd, pixFmt);
				fprintf(stderr,text);
			}
			tempRC = wglCreateContext( glHDC );
		}
		if(tempRC==0)
		{
			if(--depthPlanes==-1) 
			{
				colorPlanes-=1;
				depthPlanes=32;
			}
			if(colorPlanes==7 && *pDoublebuffer)
			{
				colorPlanes=24;
				depthPlanes=32;
				*pDoublebuffer=0;
			}
		}
	}


	if(verbose)
	{
		if(tempRC==NULL)
			fprintf(stderr,"\n\nPIXELFORMAT OF GL-Context SETTINGS failed\n");
		else {
			fprintf(stderr,"\n\nPIXELFORMAT OF GL-Context SETTINGS:\n");
			text=GetTextualPixelFormatByPFD(&pfd, pixFmt);
			fprintf(stderr,text);
		}
	}

    /* check if the context could be created */
    if( tempRC == NULL ) {
        /* fprintf(stderr, "getGC context could NOT be created \n"); */
		showLastError();
		return( 0 );
    }

    /* associated the context with the X window */
    if( wglMakeCurrent( glHDC, tempRC ) == FALSE) {
		fprintf(stderr,"\n\nCould not bind GL-Context(%d) to HDC(%d)\n",
				(int)tempRC, (int)glHDC);
		showLastError();
        wglDeleteContext( tempRC );
        return( 0 );
    } else if(verbose) {
		const char *glven, *glren, *glver, *glext, *gluver, *gluext;

        glven = glGetString(GL_VENDOR);
		glren = glGetString(GL_RENDERER);
		glver = glGetString(GL_VERSION);
		glext = glGetString(GL_EXTENSIONS);
		gluver = gluGetString(GLU_VERSION);
		gluext = gluGetString(GLU_EXTENSIONS);

		if(glven==0) glven=nav;
		if(glren==0) glren=nav;
		if(glver==0) glver=nav;
		if(glext==0) glext=nav;
		if(gluver==0) gluver=nav;
		if(gluext==0) gluext=nav;

		fprintf(stderr, "\nGL Vendor: %s\n", glven);
		fprintf(stderr, "GL Version: %s\n", glver);
		fprintf(stderr, "GL Renderer: %s\n", glren);
		fprintf(stderr, "GL EXT: %s\n\n", glext);
		fprintf(stderr, "GLU Version: %s\n", gluver);
		fprintf(stderr, "GLU EXT: %s\n\n", gluext);
	}

	glContext = tempRC;
	*pixFmtIdx=pixFmt;

    return tempRC;
}

RECT wglToolRetrieveWinSize(int *width, int *height)
{
	RECT r;

	if(GetClientRect( glWnd, &r))
	{
		*width=abs(r.right-r.left);
		*height=abs(r.bottom-r.top);
	}
	return r;
}


// Select the pixel format for a given device context
static int SetDCPixelFormat(HDC hDC, int dbuffer)
{
    int nPixelFormat=0;

	DWORD dbl_mask= 
		PFD_DRAW_TO_WINDOW        // Draw to Window (not to bitmap)
	    | PFD_SUPPORT_OPENGL	  // Support OpenGL calls in window
		;

	if(dbuffer)
		dbl_mask |= 
	    PFD_DOUBLEBUFFER          // Double buffered is optional
		;

	pfd.dwFlags = dbl_mask ;

    // Choose a pixel format that best matches that described in pfd
    nPixelFormat = ChoosePixelFormat(hDC, &pfd);

    // Set the pixel format for the device context
    return SetPixelFormat(hDC, nPixelFormat, &pfd);
}


// If necessary, creates a 3-3-2 palette for the device context listed.
static HPALETTE GetOpenGLPalette(HDC hDC)
{
    HPALETTE hRetPal = NULL;	// Handle to palette to be created
    PIXELFORMATDESCRIPTOR pfd;	// Pixel Format Descriptor
    LOGPALETTE *pPal;			// Pointer to memory for logical palette
    int nPixelFormat;			// Pixel format index
    int nColors;				// Number of entries in palette
    int i;						// Counting variable
    BYTE RedRange,GreenRange,BlueRange;
							    // Range for each color entry (7,7,and 3)


    // Get the pixel format index and retrieve the pixel format description
    nPixelFormat = GetPixelFormat(hDC);
    DescribePixelFormat(hDC, nPixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd);

    // Does this pixel format require a palette?  If not, do not create a
    // palette and just return NULL
    if(!(pfd.dwFlags & PFD_NEED_PALETTE))
	    return NULL;

    // Number of entries in palette.  8 bits yeilds 256 entries
    nColors = 1 << pfd.cColorBits;	

    // Allocate space for a logical palette structure plus all the palette entries
    pPal = (LOGPALETTE*)malloc(sizeof(LOGPALETTE) + nColors*sizeof(PALETTEENTRY));

    // Fill in palette header
    pPal->palVersion = 0x300;		// Windows 3.0
    pPal->palNumEntries = nColors; // table size

    // Build mask of all 1's.  This creates a number represented by having
    // the low order x bits set, where x = pfd.cRedBits, pfd.cGreenBits, and
    // pfd.cBlueBits.  
    RedRange = (1 << pfd.cRedBits) -1;
    GreenRange = (1 << pfd.cGreenBits) - 1;
    BlueRange = (1 << pfd.cBlueBits) -1;

    // Loop through all the palette entries
    for(i = 0; i < nColors; i++)
	    {
	    // Fill in the 8-bit equivalents for each component
	    pPal->palPalEntry[i].peRed = (i >> pfd.cRedShift) & RedRange;
	    pPal->palPalEntry[i].peRed = (unsigned char)(
		    (double) pPal->palPalEntry[i].peRed * 255.0 / RedRange);

	    pPal->palPalEntry[i].peGreen = (i >> pfd.cGreenShift) & GreenRange;
	    pPal->palPalEntry[i].peGreen = (unsigned char)(
		    (double)pPal->palPalEntry[i].peGreen * 255.0 / GreenRange);

	    pPal->palPalEntry[i].peBlue = (i >> pfd.cBlueShift) & BlueRange;
	    pPal->palPalEntry[i].peBlue = (unsigned char)(
		    (double)pPal->palPalEntry[i].peBlue * 255.0 / BlueRange);

	    pPal->palPalEntry[i].peFlags = (unsigned char) NULL;
	    }
	    

    // Create the palette
    hRetPal = CreatePalette(pPal);

    // Go ahead and select and realize the palette for this device context
    SelectPalette(hDC,hRetPal,FALSE);
    RealizePalette(hDC);

    // Free the memory used for the logical palette structure
    free(pPal);

    // Return the handle to the new palette
    return hRetPal;
}


static void
PrintMessage( const char *Format, ... )
{
    va_list ArgList;
    char Buffer[256];

    va_start(ArgList, Format);
    vsprintf(Buffer, Format, ArgList);
    va_end(ArgList);

	fprintf(stderr, Buffer);
}


static int
PixelFormatDescriptorFromDc( HDC Dc, PIXELFORMATDESCRIPTOR *Pfd )
{
    int PfdIndex;

    if ( 0 < (PfdIndex = GetPixelFormat( Dc )) )
    {
        if ( 0 < DescribePixelFormat( Dc, PfdIndex, sizeof(*Pfd), Pfd ) )
        {
            return(PfdIndex);
        }
        else
        {
            PrintMessage("Could not get a description of pixel format %d\n",
                PfdIndex );
        }
    }
    else
    {
        PrintMessage("Could not get pixel format for Dc 0x%08lX\n", Dc );
    }
    return( 0 );
}


static const char * GetTextualPixelFormatByPFD(PIXELFORMATDESCRIPTOR *ppfd, int format)
{
	static char buffer[2000];
	char line[200];

    sprintf(buffer,"Pixel format %d\n", format);
    sprintf(line,"  dwFlags - 0x%x\n", ppfd->dwFlags); 
	if (ppfd->dwFlags & PFD_DOUBLEBUFFER) { strcat(buffer, line); sprintf(line,"\tPFD_DOUBLEBUFFER\n "); }
	if (ppfd->dwFlags & PFD_STEREO) { strcat(buffer, line); sprintf(line,"\tPFD_STEREO\n "); }
	if (ppfd->dwFlags & PFD_DRAW_TO_WINDOW) { strcat(buffer, line); sprintf(line,"\tPFD_DRAW_TO_WINDOW\n "); }
    if (ppfd->dwFlags & PFD_DRAW_TO_BITMAP) { strcat(buffer, line); sprintf(line,"\tPFD_DRAW_TO_BITMAP\n "); }
    if (ppfd->dwFlags & PFD_SUPPORT_GDI) { strcat(buffer, line); sprintf(line,"\tPFD_SUPPORT_GDI\n "); }
    if (ppfd->dwFlags & PFD_SUPPORT_OPENGL) { strcat(buffer, line); sprintf(line,"\tPFD_SUPPORT_OPENGL\n "); }
    if (ppfd->dwFlags & PFD_GENERIC_FORMAT) { strcat(buffer, line); sprintf(line,"\tPFD_GENERIC_FORMAT\n "); }
    if (ppfd->dwFlags & PFD_NEED_PALETTE) { strcat(buffer, line); sprintf(line,"\tPFD_NEED_PALETTE\n "); }
    if (ppfd->dwFlags & PFD_NEED_SYSTEM_PALETTE) { strcat(buffer, line); sprintf(line,"\tPFD_NEED_SYSTEM_PALETTE\n "); }
    strcat(buffer, line); sprintf(line,"\n");
    strcat(buffer, line); sprintf(line,"  iPixelType - %d\n", ppfd->iPixelType); 
    if (ppfd->iPixelType == PFD_TYPE_RGBA) { strcat(buffer, line); sprintf(line,"\tPGD_TYPE_RGBA\n"); }
    if (ppfd->iPixelType == PFD_TYPE_COLORINDEX) { strcat(buffer, line); sprintf(line,"\tPGD_TYPE_COLORINDEX\n"); }
    strcat(buffer, line); sprintf(line,"  cColorBits - %d\n", ppfd->cColorBits);
    strcat(buffer, line); sprintf(line,"  cRedBits - %d\n", ppfd->cRedBits);
    strcat(buffer, line); sprintf(line,"  cRedShift - %d\n", ppfd->cRedShift);
    strcat(buffer, line); sprintf(line,"  cGreenBits - %d\n", ppfd->cGreenBits);
    strcat(buffer, line); sprintf(line,"  cGreenShift - %d\n", ppfd->cGreenShift);
    strcat(buffer, line); sprintf(line,"  cBlueBits - %d\n", ppfd->cBlueBits);
    strcat(buffer, line); sprintf(line,"  cBlueShift - %d\n", ppfd->cBlueShift);
    strcat(buffer, line); sprintf(line,"  cAlphaBits - %d\n", ppfd->cAlphaBits);
    strcat(buffer, line); sprintf(line,"  cAlphaShift - 0x%x\n", ppfd->cAlphaShift);
    strcat(buffer, line); sprintf(line,"  cAccumBits - %d\n", ppfd->cAccumBits);
    strcat(buffer, line); sprintf(line,"  cAccumRedBits - %d\n", ppfd->cAccumRedBits);
    strcat(buffer, line); sprintf(line,"  cAccumGreenBits - %d\n", ppfd->cAccumGreenBits);
    strcat(buffer, line); sprintf(line,"  cAccumBlueBits - %d\n", ppfd->cAccumBlueBits);
    strcat(buffer, line); sprintf(line,"  cAccumAlphaBits - %d\n", ppfd->cAccumAlphaBits);
    strcat(buffer, line); sprintf(line,"  cDepthBits - %d\n", ppfd->cDepthBits);
    strcat(buffer, line); sprintf(line,"  cStencilBits - %d\n", ppfd->cStencilBits);
    strcat(buffer, line); sprintf(line,"  cAuxBuffers - %d\n", ppfd->cAuxBuffers);
    strcat(buffer, line); sprintf(line,"  iLayerType - %d\n", ppfd->iLayerType);
    strcat(buffer, line); sprintf(line,"  bReserved - %d\n", ppfd->bReserved);
    strcat(buffer, line); sprintf(line,"  dwLayerMask - 0x%x\n", ppfd->dwLayerMask);
    strcat(buffer, line); sprintf(line,"  dwVisibleMask - 0x%x\n", ppfd->dwVisibleMask);
    strcat(buffer, line); sprintf(line,"  dwDamageMask - 0x%x\n", ppfd->dwDamageMask);
	strcat(buffer, line); 
	return buffer;
}

static const char * GetTextualPixelFormatByHDC(HDC hdc)
{
    PIXELFORMATDESCRIPTOR pfd, *ppfd;
    int format;

    ppfd   = &pfd;
    format = PixelFormatDescriptorFromDc( hdc, ppfd );

	return GetTextualPixelFormatByPFD(ppfd, format);
}

